package org.processmining.plugins.workloadperiod.filter;

import java.util.ArrayList;
import java.util.Collections;
import java.util.Date;
import java.util.Iterator;
import java.util.List;
import java.util.ListIterator;
import java.util.Set;
import java.util.TreeSet;

import javax.swing.JTable;

import org.processmining.framework.util.Pair;
import org.processmining.framework.util.collection.ComparablePair;
import org.processmining.plugins.workloadperiod.main.WLSelectColumn;

/**
 * Workload Analysis Intervals
 * 
 * Creates Intervals based on the Start Times for all the Resources in the given
 * Table
 * 
 * @author jnakatumba
 * 
 */

public class WLAllIntervals {

	private WLSelectColumn selectCol;

	private String refEventType;
	private String loadEventType;

	private Set<ComparablePair<Date, Date>> horizon;
	private ComparablePair<Date, Date> dateShifts;
	private ComparablePair<Date, Date> shiftSt;

	private List<Double> noOfCases;
	private List<Date> startTimesList, compTimesList;

	private List<ComparablePair<Date, Date>> myShift;
	private List<ComparablePair<Date, Long>> compDurationList, startDurationList;
	private List<Pair<Date, Double>> noOfCasesPerShift;

	private JTable filterTable;

	/**
	 * Based on the time intervals built
	 * 
	 * @param intervals
	 *            The time intervals built based on the hours per horizon
	 *            specified.
	 * 
	 */
	public WLAllIntervals(WLTimeInterval intervals,
			List<ComparablePair<Date, Long>> startDurationLists, List<ComparablePair<Date, Long>> compDurationLists,
			JTable table, String refEvents, String loadEvents) {

		this.filterTable = table;
		this.refEventType = refEvents;
		this.loadEventType = loadEvents;
		this.startDurationList = startDurationLists;
		this.compDurationList = compDurationLists;

		startTimesList = new ArrayList<Date>();
		compTimesList = new ArrayList<Date>();
		horizon = new TreeSet<ComparablePair<Date, Date>>();
		dateShifts = new ComparablePair<Date, Date>(null, null);
		shiftSt = new ComparablePair<Date, Date>(null, null);
		myShift = new ArrayList<ComparablePair<Date, Date>>();
		noOfCases = new ArrayList<Double>();

		selectCol = new WLSelectColumn(filterTable);

		startTimesList = selectCol.getStartTimes();
		compTimesList = selectCol.getCompleteTimes();

		Collections.sort(startTimesList);
		Collections.sort(compTimesList);
		Collections.sort(startDurationList);
		Collections.sort(compDurationList);

		if (refEventType.equals("Start")) {

			for (ListIterator<ComparablePair<Date, Long>> currentOne = startDurationList.listIterator(); currentOne
					.hasNext();) {
				if (currentOne.hasNext() == true) {
					ComparablePair<Date, Long> op = currentOne.next();
					Date firstOne = op.getFirst();
					shiftSt = intervals.getShifts(firstOne);
					horizon.add(shiftSt);

				}
			}

		}
		/**
		 * get shifts based on the complete event types
		 */
		if (refEventType.equals("Complete")) {
			for (ListIterator<ComparablePair<Date, Long>> currentOne = compDurationList.listIterator(); currentOne
					.hasNext();) {
				if (currentOne.hasNext() == true) {
					ComparablePair<Date, Long> op = currentOne.next();
					Date firstOne = op.getFirst();

					shiftSt = intervals.getShifts(firstOne);
					horizon.add(shiftSt);

				}
			}
		}
		/**
		 * get shifts based on both start and complete event types
		 */
		if (refEventType.equals("StartComplete")) {
			for (ListIterator<ComparablePair<Date, Long>> currentOne = startDurationList.listIterator(); currentOne
					.hasNext();) {
				if (currentOne.hasNext() == true) {
					ComparablePair<Date, Long> op = currentOne.next();
					Date firstOne = op.getFirst();
					shiftSt = intervals.getShifts(firstOne);
					horizon.add(shiftSt);

				}
			}

			for (ListIterator<ComparablePair<Date, Long>> currentOne = compDurationList.listIterator(); currentOne
					.hasNext();) {
				if (currentOne.hasNext() == true) {
					ComparablePair<Date, Long> op = currentOne.next();
					Date firstOne = op.getFirst();
					shiftSt = intervals.getShifts(firstOne);
					horizon.add(shiftSt);

				}
			}
		}

		start();

		startTimesList.clear();
		compTimesList.clear();
	}

	private void start() {
		getShift();
		displayResource();

	}

	@SuppressWarnings("unchecked")
	private void getShift() {
		ComparablePair<Date, Date>[] allHorizon = new ComparablePair[horizon.size()];

		allHorizon = horizon.toArray(allHorizon);
		//horModel.addColumn("Shift-Period", allHorizon);
	}

	private void displayResource() {
		if (loadEventType.equals("Start")) {
			shiftsBasedOnStart();
			noOfCasesPerShift();
		}
		if (loadEventType.equals("Complete")) {
			shiftsBasedOnComplete();
			noOfCasesPerShift();
		}
		if (loadEventType.equals("StartComplete")) {
			shiftsBasedOnStart();
			shiftsBasedOnComplete();
			noOfCasesPerShift();

		}

	}

	/**
	 * This method returns the shifts within which the complete dates lie.
	 * 
	 * @return A list with the different shifts per complete date
	 */

	@SuppressWarnings("unchecked")
	private List<ComparablePair<Date, Date>> shiftsBasedOnComplete() {
		myShift.clear();
		Object[] horizons = horizon.toArray();
		for (Iterator compRef = compTimesList.iterator(); compRef.hasNext();) {
			if (compRef.hasNext() == true) {
				Date cDate = (Date) compRef.next();
				for (int horIndex = 0; horIndex < horizons.length; horIndex++) {
					dateShifts = (ComparablePair<Date, Date>) horizons[horIndex];
					Date firstDate = dateShifts.getFirst();
					Date secondDate = dateShifts.getSecond();

					if (cDate.after(firstDate) && cDate.before(secondDate)) {

						myShift.add(dateShifts);
						//break now;

					}

				}
			}
		}
		return myShift;
	}

	/**
	 * This method returns the shifts within which the start dates lie.
	 * 
	 * @return A list with the different shifts per start date
	 */

	@SuppressWarnings("unchecked")
	private List<ComparablePair<Date, Date>> shiftsBasedOnStart() {
		Object[] horizons = horizon.toArray();
		for (Iterator compRef = startTimesList.iterator(); compRef.hasNext();) {
			if (compRef.hasNext() == true) {
				Date cDate = (Date) compRef.next();
				for (int horIndex = 0; horIndex < horizons.length; horIndex++) {
					dateShifts = (ComparablePair<Date, Date>) horizons[horIndex];
					Date firstDate = dateShifts.getFirst();
					Date secondDate = dateShifts.getSecond();

					if (cDate.after(firstDate) && cDate.before(secondDate)) {

						myShift.add(dateShifts);
						//break now;

					}

				}

			}
		}
		return myShift;

	}

	/**
	 * Based on the list with the shifts representing the start and complete
	 * dates, this method then returns the number of cases started and/or
	 * completed per shift
	 * 
	 * @return A list of number of cases per shift
	 */

	@SuppressWarnings("unchecked")
	private void noOfCasesPerShift() {
		int noShift = 0;
		Collections.sort(myShift);
		noOfCasesPerShift = new ArrayList<Pair<Date, Double>>();
		ComparablePair<Date, Date> shiftPair = new ComparablePair<Date, Date>(null, null);
		ComparablePair<Date, Date>[] allHorizon = new ComparablePair[horizon.size()];
		allHorizon = horizon.toArray(allHorizon);
		Object[] horizonArray = horizon.toArray();

		for (int horIndex = 0; horIndex < horizonArray.length; horIndex++) {
			shiftPair = (ComparablePair<Date, Date>) horizonArray[horIndex];
			while (myShift.contains(shiftPair)) {
				noShift = noShift + 1;
				myShift.remove(shiftPair);
			}

			Date firstOne = shiftPair.getFirst();
			Double noShifts = Double.valueOf(noShift);
			Pair<Date, Double> e = new Pair<Date, Double>(firstOne, noShifts);

			noOfCasesPerShift.add(e);
			noShift = 0;

		}
		getNumberOfCases();
	}

	private List<Double> getNumberOfCases() {

		Date cDate = null;
		for (Iterator<ComparablePair<Date, Date>> current = horizon.iterator(); current.hasNext();) {
			if (current.hasNext() == true) {
				ComparablePair<Date, Date> currentPairShift = current.next();
				Date firstDate = currentPairShift.getFirst();

				second: for (int shiftCount = 0; shiftCount < noOfCasesPerShift.size(); shiftCount++) {
					Pair<Date, Double> currentPair = noOfCasesPerShift.get(shiftCount);
					cDate = currentPair.getFirst();
					Double secondOne = currentPair.getSecond();
					if (firstDate.equals(cDate)) {
						noOfCases.add(secondOne);
						break second;
					}
				}
				if (firstDate.equals(cDate) == false)
					noOfCases.add(0.0);

			}
		}

		return noOfCases;

	}

	public List<Double> getCases() {
		return noOfCases;
	}

}
