package org.processmining.plugins.workloadperiod.filter;

import java.util.ArrayList;
import java.util.Date;
import java.util.Iterator;
import java.util.List;
import java.util.ListIterator;
import java.util.Set;
import java.util.TreeSet;

import org.processmining.framework.util.collection.ComparablePair;

/**
 * WorkLoad Analysis Time Intervals
 * 
 * Generates the number of cases executed per shift period and the corresponding
 * service times
 * 
 * @author jnakatumba
 * 
 * 
 */

public class WLIntervalsOne {
	private Set<ComparablePair<Date, Date>> horizon;

	private List<Date> startTimesList, compTimesList;
	private List<ComparablePair<Date, Date>> myShift;
	private List<ComparablePair<Date, Long>> startDurationList;
	private List<ComparablePair<Date, Long>> compDurationList;
	private List<Double> noOfCases = new ArrayList<Double>();
	private List<Double> serviceTimes = new ArrayList<Double>();

	private ComparablePair<Date, Date> dateShifts;
	private ComparablePair<Date, Date> shiftSt;

	private String refEvents;
	private String loadEvents;

	/**
	 * Based on the time intervals built
	 * 
	 * @param intervals
	 *            The time intervals built based on the hours per horizon
	 *            specified.
	 * 
	 */

	public WLIntervalsOne(WLTimeInterval intervals, List<Date> startTimesLists,
			List<ComparablePair<Date, Long>> startDurationList2, List<ComparablePair<Date, Long>> compDurationList2,
			List<Date> compTimesList2, String refEvent, String loadEvent) {

		this.startTimesList = startTimesLists;
		this.startDurationList = startDurationList2;
		this.compDurationList = compDurationList2;
		this.compTimesList = compTimesList2;
		this.refEvents = refEvent;
		this.loadEvents = loadEvent;

		horizon = new TreeSet<ComparablePair<Date, Date>>();
		dateShifts = new ComparablePair<Date, Date>(null, null);
		shiftSt = new ComparablePair<Date, Date>(null, null);
		myShift = new ArrayList<ComparablePair<Date, Date>>();

		if (refEvents != null) {
			if (refEvents.equals("Start")) {
				for (ListIterator<ComparablePair<Date, Long>> currentOne = startDurationList.listIterator(); currentOne
						.hasNext();) {
					if (currentOne.hasNext() == true) {
						ComparablePair<Date, Long> op = currentOne.next();
						Date firstOne = op.getFirst();
						Long secondOne = op.getSecond();
						shiftSt = intervals.getShifts(firstOne);
						if (horizon.contains(shiftSt) == false) {
							horizon.add(shiftSt);
							Double dDur = Double.valueOf(secondOne);
							serviceTimes.add(dDur);

						}
					}
				}
				getShiftEvents();

			} else if (refEvents.equals("Complete")) {
				for (ListIterator<ComparablePair<Date, Long>> currentOne = compDurationList.listIterator(); currentOne
						.hasNext();) {
					if (currentOne.hasNext() == true) {
						ComparablePair<Date, Long> op = currentOne.next();
						Date firstOne = op.getFirst();
						Long secondOne = op.getSecond();
						shiftSt = intervals.getShifts(firstOne);
						if (horizon.contains(shiftSt) == false) {
							horizon.add(shiftSt);
							Double dDur = Double.valueOf(secondOne);
							serviceTimes.add(dDur);

						}
					}
				}

				getShiftEvents();

			} else if (refEvents.equals("StartComplete")) {
				for (ListIterator<ComparablePair<Date, Long>> currentOne = startDurationList.listIterator(); currentOne
						.hasNext();) {
					if (currentOne.hasNext() == true) {
						if (currentOne.hasNext() == true) {
							ComparablePair<Date, Long> op = currentOne.next();
							Date firstOne = op.getFirst();
							Long secondOne = op.getSecond();
							shiftSt = intervals.getShifts(firstOne);
							if (horizon.contains(shiftSt) == false) {
								horizon.add(shiftSt);
								Double dDur = Double.valueOf(secondOne);
								serviceTimes.add(dDur);

							}
						}
					}
				}

				for (ListIterator<ComparablePair<Date, Long>> currentOne = compDurationList.listIterator(); currentOne
						.hasNext();) {
					if (currentOne.hasNext() == true) {
						ComparablePair<Date, Long> op = currentOne.next();
						Date firstOne = op.getFirst();
						Long secondOne = op.getSecond();
						shiftSt = intervals.getShifts(firstOne);
						if (horizon.contains(shiftSt) == false) {
							horizon.add(shiftSt);
							Double dDur = Double.valueOf(secondOne);
							serviceTimes.add(dDur);

						}
					}
				}

				getShiftEvents();

			}

			noOfCasesPerShift();

		}
	}

	private void getShiftEvents() {
		if (loadEvents != null) {
			if (loadEvents.equals("Start")) {
				shiftsBasedOnStart();
			} else if (loadEvents.equals("Complete")) {
				shiftsBasedOnComplete();
			} else if (loadEvents.equals("StartComplete")) {
				shiftsBasedOnStart();
				shiftsBasedOnComplete();
			}
		}
	}

	/**
	 * This method returns the shifts within which the complete dates lie.
	 * 
	 * @return A list with the different shifts per complete date
	 */

	@SuppressWarnings("unchecked")
	private List<ComparablePair<Date, Date>> shiftsBasedOnComplete() {

		Object[] horizons = horizon.toArray();

		for (Iterator compRef = compTimesList.iterator(); compRef.hasNext();) {
			if (compRef.hasNext() == true) {
				Date cDate = (Date) compRef.next();
				for (int horIndex = 0; horIndex < horizons.length; horIndex++) {
					dateShifts = (ComparablePair<Date, Date>) horizons[horIndex];
					Date firstDate = dateShifts.getFirst();
					Date secondDate = dateShifts.getSecond();

					if (cDate.after(firstDate) && cDate.before(secondDate)) {

						myShift.add(dateShifts);
						//break now;

					}

				}
			}
		}
		return myShift;
	}

	/**
	 * This method returns the shifts within which the start dates lie.
	 * 
	 * @return A list with the different shifts per start date
	 */

	@SuppressWarnings("unchecked")
	private List<ComparablePair<Date, Date>> shiftsBasedOnStart() {

		Object[] horizons = horizon.toArray();
		for (Iterator compRef = startTimesList.iterator(); compRef.hasNext();) {
			if (compRef.hasNext() == true) {
				Date cDate = (Date) compRef.next();
				for (int horIndex = 0; horIndex < horizons.length; horIndex++) {
					dateShifts = (ComparablePair<Date, Date>) horizons[horIndex];
					Date firstDate = dateShifts.getFirst();
					Date secondDate = dateShifts.getSecond();

					if (cDate.after(firstDate) && cDate.before(secondDate)) {

						myShift.add(dateShifts);
						//break now;

					}

				}

			}
		}
		return myShift;

	}

	@SuppressWarnings("unchecked")
	private List<Double> noOfCasesPerShift() {
		int noShift = 0;

		ComparablePair<Date, Date> shiftPair = new ComparablePair<Date, Date>(null, null);
		ComparablePair<Date, Date>[] allHorizon = new ComparablePair[horizon.size()];
		noOfCases.clear();
		allHorizon = horizon.toArray(allHorizon);
		Object[] horizonArray = horizon.toArray();

		for (int horIndex = 0; horIndex < horizonArray.length; horIndex++) {
			shiftPair = (ComparablePair<Date, Date>) horizonArray[horIndex];
			while (myShift.contains(shiftPair)) {
				noShift = noShift + 1;
				myShift.remove(shiftPair);
			}
			Double noShifts = Double.valueOf(noShift);
			noOfCases.add(noShifts);
			noShift = 0;
		}

		return noOfCases;
	}

	/**
	 * Gets the Number of cases executed per shift Period
	 * 
	 * @return List with the Number of cases
	 */
	public List<Double> getCases() {
		return noOfCases;
	}

	/**
	 * Gets the Service Times per shift Period
	 * 
	 * @return List with the service Times
	 */

	public List<Double> getServiceTime() {
		return serviceTimes;

	}

}
