package org.processmining.plugins.workloadperiod.filter;

import java.util.Calendar;
import java.util.Date;

import org.processmining.framework.util.collection.ComparablePair;

/**
 * WorkLoad Analysis Intervals
 * 
 * Creates the shift period based on the hours per shift.
 * 
 * @author jnakatumba
 * 
 * 
 */

public class WLTimeInterval {

	private int horizonperiod;
	private Date startDate;
	private Date endDate;

	/**
	 * Build the time intervals based on the value provided for the parameter
	 * hoursPerHorizon.
	 * 
	 * @param Number
	 *            of hours per horizon of a resource.
	 */
	public WLTimeInterval(int hoursPerHorizon) {
		this.horizonperiod = hoursPerHorizon;
	}

	/**
	 * Gets the shift periods
	 * 
	 * @param timeStamp
	 *            Specific times
	 * @return The start and the end of the shift period
	 */

	public ComparablePair<Date, Date> getShifts(Date timeStamp) {
		getShiftsForward(timeStamp);
		getShiftsBackward(timeStamp);
		return new ComparablePair<Date, Date>(startDate, endDate);

	}

	/**
	 * Gets the forward shiftPeriod
	 * 
	 * @param timeStamp
	 * @return 
	 * time based on the a specific number of hours per shift period
	 */

	private Date getShiftsForward(Date timeStamp) {
		Calendar myCalendar = Calendar.getInstance();
		myCalendar.setTime(timeStamp);

		myCalendar.add(Calendar.HOUR_OF_DAY, horizonperiod);
		myCalendar.add(Calendar.SECOND, -1);
		endDate = myCalendar.getTime();
		return endDate;

	}
	/**
	 * Gets the backward shift period
	 * @param timeStamp
	 * @return
	 * 	time period based on a specific number of hours per shift period
	 */

	private Date getShiftsBackward(Date timeStamp) {
		Calendar myCalendar = Calendar.getInstance();
		myCalendar.setTime(timeStamp);

		myCalendar.add(Calendar.HOUR_OF_DAY, -horizonperiod);
		myCalendar.add(Calendar.SECOND, -1);
		startDate = myCalendar.getTime();
		return startDate;

	}

}
