package org.processmining.plugins.workloadperiod.input;

import java.util.ArrayList;
import java.util.Collection;
import java.util.Date;
import java.util.List;
import java.util.Set;
import java.util.TreeSet;

import javax.swing.JTable;
import javax.swing.table.TableModel;

import org.deckfour.xes.model.XLog;
import org.processmining.framework.plugin.PluginContext;
import org.processmining.framework.plugin.Progress;
import org.processmining.plugins.workloadperiod.main.WLChoice;
import org.processmining.plugins.workloadperiod.main.WLMainTable;
import org.processmining.plugins.workloadperiod.main.WLSettings;
import org.processmining.plugins.workloadperiod.main.WLSimpleOutput;
import org.processmining.plugins.workloadperiod.main.WLSimpleTypes;

/**
 * Workload Analysis Main class
 * 
 * @author jnakatumba
 * 
 * 
 */

public class WLAnalysis {

	/**
	 * The context of this miner.
	 */
	private PluginContext context;

	private Collection<WLSimpleTypes> simpleAnalysis;

	private WLMainTable mainReadLog;
	private WLChoice displayChoice;
	private String wlCalc;
	private WLSimpleOutput output;
	private XLog log;

	/**
	 * Creates a miner, given its context.
	 * 
	 * @param context
	 */
	public WLAnalysis(final PluginContext context) {
		this.context = context;
	}

	public WLAnalysis(PluginContext context, XLog log) {
		this.context = context;
		this.log = log;

	}

	public WLSimpleOutput mine(final WLSettings input) {
		final Progress progress = context.getProgress();
		progress.setMinimum(0);
		progress.setMaximum(5);
		progress.setIndeterminate(false);
		Set<String> resourceList = new TreeSet<String>();
		List<Date> startDatesList = new ArrayList<Date>();

		context.getProgress().setMinimum(0);
		context.getProgress().setIndeterminate(false);

		mainReadLog = new WLMainTable(log);
		TableModel mainModel = mainReadLog.getMainTable();
		JTable mainTable = new JTable(mainModel);

		// This is the one doing the hard work.
		context.getProgress().inc();

		String resNames;
		for (Integer colIndex = 0; colIndex < mainTable.getColumnCount(); colIndex++) {
			if (mainTable.getColumnName(colIndex).equals("Resource Names")) {
				for (Integer rowIndex = 0; rowIndex < mainTable.getRowCount(); rowIndex++) {
					Object value = mainTable.getValueAt(rowIndex, colIndex);
					if (value != null) {
						resNames = (String) value;
						resourceList.add(resNames);
					}
				}
			}

		}

		for (Integer colIndex = 0; colIndex < mainTable.getColumnCount(); colIndex++) {
			if (mainTable.getColumnName(colIndex).equals("Start Times")) {
				for (Integer rowIndex = 0; rowIndex < mainTable.getRowCount(); rowIndex++) {
					Object value = mainTable.getValueAt(rowIndex, colIndex);
					if (value != null) {
						Date date = (Date) value;
						startDatesList.add(date);
					}
				}
			}

		}

		if (resourceList.isEmpty() == true || startDatesList.isEmpty() == true) {
			output = new WLSimpleOutput(context);
			StringBuffer message = new StringBuffer();
			message.append("Sorry!!! The plug-in can not give any results because of the following issues: ");
			message.append("\n");
			message.append("\n");
			message.append("\n");
			message.append("a) The event log does not have resource information.");
			message.append("\n ");
			message.append("\n");
			message.append("\n");
			message.append("\n");
			message.append("b) The event log lacks the start events needed for calculating the service times.");
			output.setDisplayMessage(message);
		} else {

			displayChoice = new WLChoice(input, mainReadLog);

			wlCalc = input.getWorkLoadType();

			if (wlCalc == null) {
				wlCalc = "How Busy";
			}

			if (wlCalc.equals("How Busy") == true) {
				output = new WLSimpleOutput(context);
				String dChoice = input.getDisplayChoice();
				if (dChoice == null) {
					dChoice = "Simple Regression";
				}
				output.setDChoice(dChoice);
				output.setWLType(wlCalc);

				simpleAnalysis = input.getSimpleDisplay();
				if (simpleAnalysis.isEmpty() == true) {
					simpleAnalysis.add(WLSimpleTypes.ONERESOURCEONETASK);
				}

				if (dChoice.equals("Simple Regression")) {
					if (simpleAnalysis.contains(WLSimpleTypes.ONERESOURCEONETASK)) {
						JTable dTable = displayChoice.getShiftTables();

						output.setTables(WLSimpleTypes.ONERESOURCEONETASK, dTable);

					}
					if (simpleAnalysis.contains(WLSimpleTypes.ALLTASKSONERESOURCE)) {

						JTable dTable = displayChoice.getShiftTables1();

						output.setTables(WLSimpleTypes.ALLTASKSONERESOURCE, dTable);

					}

					if (simpleAnalysis.contains(WLSimpleTypes.ALLRESOURCESTASKS)) {
						JTable dTable = displayChoice.getShiftTables3();
						output.setTables1(WLSimpleTypes.ALLRESOURCESTASKS, dTable);

					}
					if (simpleAnalysis.contains(WLSimpleTypes.ALLRESOURCESONETASK)) {
						JTable dTable = displayChoice.getShiftTables2();
						output.setTables(WLSimpleTypes.ALLRESOURCESONETASK, dTable);
					}

				}

			} else {
				output = new WLSimpleOutput(context);
				output.setWLType(wlCalc);

				simpleAnalysis = input.getSimpleDisplay();

				if (simpleAnalysis.contains(WLSimpleTypes.ONERESOURCEONETASK)) {
					JTable dTable = displayChoice.getShiftTables();
					output.setTables(WLSimpleTypes.ONERESOURCEONETASK, dTable);
				}
				if (simpleAnalysis.contains(WLSimpleTypes.ALLTASKSONERESOURCE)) {
					JTable dTable = displayChoice.getShiftTables1();
					output.setTables(WLSimpleTypes.ALLTASKSONERESOURCE, dTable);
				}

				if (simpleAnalysis.contains(WLSimpleTypes.ALLRESOURCESTASKS)) {
					JTable dTable = displayChoice.getShiftTables3();
					output.setTables1(WLSimpleTypes.ALLRESOURCESTASKS, dTable);
				}
				if (simpleAnalysis.contains(WLSimpleTypes.ALLRESOURCESONETASK)) {
					JTable dTable = displayChoice.getShiftTables2();
					output.setTables(WLSimpleTypes.ALLRESOURCESONETASK, dTable);
				}
			}

			context.getProgress().setCaption("Constructing Workload analysis..");
		}
		return output;
	}

}
