package org.processmining.plugins.workloadperiod.input;

import org.deckfour.xes.model.XLog;
import org.processmining.contexts.uitopia.UIPluginContext;
import org.processmining.contexts.uitopia.annotations.UITopiaVariant;
import org.processmining.framework.plugin.PluginContext;
import org.processmining.framework.plugin.annotations.Plugin;
import org.processmining.framework.plugin.annotations.PluginVariant;
import org.processmining.plugins.workloadperiod.main.WLSettings;
import org.processmining.plugins.workloadperiod.main.WLSimpleOutput;
import org.processmining.plugins.workloadperiod.ui.WLMinerUI;

/**
 * Workload Analysis Plugin
 * 
 * 
 * @author jnakatumba
 * 
 */

@Plugin(name = "Workload Analysis", parameterLabels = { "Log" }, returnLabels = { "WorkloadAnalysis" }, returnTypes = { WLSimpleOutput.class }, userAccessible = true)
public class WLPlugin {
	/**
	 * The analysis plugin. It asks the user for the filter dimension and
	 * generates the appropriate table. If there is no input from the user, then
	 * the result is based on the default settings
	 * 
	 * @param context
	 *            The UI Context
	 * @param log
	 *            The given log
	 * @return The analysis Results
	 */
	@UITopiaVariant(affiliation = UITopiaVariant.EHV, author = "J. Nakatumba", email = "jnakatum@tue.nl")
	@PluginVariant(variantLabel = "Select Options to use", requiredParameterLabels = { 0 })
	public static WLSimpleOutput main(final UIPluginContext context, final XLog log) {

		WLSettings input = new WLSettings();
		WLMinerUI userInt = new WLMinerUI(context);
		return userInt.mine(log, input);
	}

	
	@PluginVariant(variantLabel = "Default Options", requiredParameterLabels = { 0 })
	public static WLSimpleOutput main(final PluginContext context, final XLog log) {
		WLSettings input = new WLSettings();
		return doWLMining(context, log, input);
	}

	private static WLSimpleOutput doWLMining(PluginContext context, XLog log, WLSettings input) {
		WLAnalysis miner = new WLAnalysis(context, log);
		WLSimpleOutput output = miner.mine(input);
		return output;

	}

}
