package org.processmining.plugins.workloadperiod.main;

import java.util.ArrayList;
import java.util.Collection;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.Set;
import java.util.TreeSet;

import javax.swing.JPanel;
import javax.swing.JTable;
import javax.swing.table.DefaultTableModel;

import org.processmining.framework.util.Pair;
import org.processmining.plugins.workloadqueue.WQMain;

/**
 * Workload choice class
 * 
 * @author jnakatumba
 * 
 */
public class WLChoice {

	public WLSimple simpleDisplay;
	private WQMain queMain;

	private Map<StringBuffer, Pair<Pair<Double, Double>, Pair<Double, Double>>> oneResActDetails;
	private Map<String, Pair<Pair<Double, Double>, Pair<Double, Double>>> oneResAllActDetails;
	private Map<String, Pair<Pair<Double, Double>, Pair<Double, Double>>> allActOneResDetails;
	private Pair<Pair<Double, Double>, Pair<Double, Double>> allActResDetails;
	private List<Double> rList, interceptList, rSquareList, slopeList;
	private Set<StringBuffer> allResultsSt;
	private List<Map<String, JPanel>> namePanelList;
	private List<Map<String, JPanel>> namePanelList1;

	private Collection<Pair<Pair<Double, Double>, Pair<Double, Double>>> setFromDetails;
	private Collection<WLSimpleTypes> simpleRegression;

	private String dChoice;
	private String wlCalc;

	private DefaultTableModel oneResActModel;
	private JTable dTable2;
	private JTable dTable3;
	private JTable dTable4;
	private JTable dTable1;
	private JPanel jPanel2;
	private JPanel jPanel4;
	private JPanel jPanel5;

	public WLChoice(WLSettings settings, WLMainTable mainTable) {
		dChoice = settings.getDisplayChoice();
		allResultsSt = new TreeSet<StringBuffer>();
		simpleRegression = settings.getSimpleDisplay();

		rList = new ArrayList<Double>();
		interceptList = new ArrayList<Double>();
		rSquareList = new ArrayList<Double>();
		slopeList = new ArrayList<Double>();
		setFromDetails = new TreeSet<Pair<Pair<Double, Double>, Pair<Double, Double>>>();

		wlCalc = settings.getWorkLoadType();
		if (wlCalc == null) {
			wlCalc = "How Busy";
		}
		if (wlCalc.equals("How Busy") == true) {
			if (dChoice == null) {
				dChoice = "Simple Regression";
			}
			if (dChoice.equals("Simple Regression")) {
				simpleDisplay = new WLSimple(settings, mainTable);
				if (simpleRegression.isEmpty() == true) {
					simpleRegression.add(WLSimpleTypes.ONERESOURCEONETASK);
				}
				if (simpleRegression.contains(WLSimpleTypes.ONERESOURCEONETASK)) {
					oneResActDetails = simpleDisplay.outputOneResActSt();
					if (oneResActDetails.size() != 0) {

						allResultsSt.clear();
						allResultsSt = oneResActDetails.keySet();
						setFromDetails.clear();
						setFromDetails = oneResActDetails.values();
						oneResActModel = new DefaultTableModel();
						oneResActModel.addColumn("Names", allResultsSt.toArray());

						getDetails();
						dTable1 = new JTable(oneResActModel);
					}
				}
				if (simpleRegression.contains(WLSimpleTypes.ALLTASKSONERESOURCE)) {
					allActOneResDetails = simpleDisplay.ouputOneResAllActSt();
					if (allActOneResDetails.size() != 0) {
						Set<String> names = allActOneResDetails.keySet();
						setFromDetails.clear();
						setFromDetails = allActOneResDetails.values();
						oneResActModel = new DefaultTableModel();
						oneResActModel.addColumn("Resource Names", names.toArray());

						getDetails();
						dTable2 = new JTable(oneResActModel);
					}
				}

				if (simpleRegression.contains(WLSimpleTypes.ALLRESOURCESONETASK)) {
					oneResAllActDetails = simpleDisplay.outputAllResOneActSt();

					if (oneResAllActDetails.size() != 0) {
						Set<String> names = oneResAllActDetails.keySet();
						setFromDetails.clear();
						setFromDetails = oneResAllActDetails.values();
						oneResActModel = new DefaultTableModel();
						oneResActModel.addColumn("Activity Names", names.toArray());

						getDetails();
						dTable3 = new JTable(oneResActModel);
					}
				}
				if (simpleRegression.contains(WLSimpleTypes.ALLRESOURCESTASKS)) {

					allActResDetails = simpleDisplay.outputAllResActStart();
					oneResActModel = new DefaultTableModel();

					getAllDetails();
					dTable4 = new JTable(oneResActModel);

				}

			}
		} else {
			queMain = new WQMain(settings, mainTable);
			if (simpleRegression.contains(WLSimpleTypes.ONERESOURCEONETASK)) {
				oneResActDetails = queMain.ouputOneResOneActSt();
				allResultsSt.clear();
				allResultsSt = oneResActDetails.keySet();
				setFromDetails.clear();
				setFromDetails = oneResActDetails.values();
				oneResActModel = new DefaultTableModel();
				oneResActModel.addColumn("Names", allResultsSt.toArray());

				getDetails();
				dTable1 = new JTable(oneResActModel);

			}
			if (simpleRegression.contains(WLSimpleTypes.ALLTASKSONERESOURCE)) {
				allActOneResDetails = queMain.ouputOneResAllActSt();
				Set<String> names = allActOneResDetails.keySet();
				setFromDetails.clear();
				setFromDetails = allActOneResDetails.values();
				oneResActModel = new DefaultTableModel();
				oneResActModel.addColumn("Resource Names", names.toArray());

				getDetails();
				dTable2 = new JTable(oneResActModel);

			}

			if (simpleRegression.contains(WLSimpleTypes.ALLRESOURCESONETASK)) {
				oneResAllActDetails = queMain.ouputAllResOneActSt();
				Set<String> names = oneResAllActDetails.keySet();
				setFromDetails.clear();
				setFromDetails = oneResAllActDetails.values();
				oneResActModel = new DefaultTableModel();
				oneResActModel.addColumn("Activity Names", names.toArray());

				getDetails();
				dTable3 = new JTable(oneResActModel);

			}
			if (simpleRegression.contains(WLSimpleTypes.ALLRESOURCESTASKS)) {
				allActResDetails = queMain.ouputAllResAllActSt();
				oneResActModel = new DefaultTableModel();
				if (allActResDetails != null) {
					getAllDetails();
					dTable4 = new JTable(oneResActModel);

				}
			}
		}

	}

	@SuppressWarnings("unchecked")
	private void getDetails() {
		for (Iterator compRef = setFromDetails.iterator(); compRef.hasNext();) {
			if (compRef.hasNext() == true) {
				Pair<Pair<Double, Double>, Pair<Double, Double>> values = (Pair<Pair<Double, Double>, Pair<Double, Double>>) compRef
						.next();
				if (values != null) {
					Pair<Double, Double> firstOne = values.getFirst();
					Pair<Double, Double> secondOne = values.getSecond();

					Double corrCoef = firstOne.getFirst();
					Double rSquare = firstOne.getSecond();

					Double intercept = secondOne.getFirst();
					Double slope = secondOne.getSecond();

					rList.add(corrCoef);
					rSquareList.add(rSquare);
					interceptList.add(intercept);
					slopeList.add(slope);
				}

			}
		}
		displayOneActResModel();
		rList.clear();
		rSquareList.clear();
		interceptList.clear();
		slopeList.clear();

	}

	private void displayOneActResModel() {

		oneResActModel.addColumn("r", rList.toArray());
		oneResActModel.addColumn("R-Square", rSquareList.toArray());
		oneResActModel.addColumn("Intercept", interceptList.toArray());
		oneResActModel.addColumn("Slope", slopeList.toArray());

	}

	private void getAllDetails() {
		Pair<Double, Double> firstOne = allActResDetails.getFirst();
		Pair<Double, Double> secondOne = allActResDetails.getSecond();
		if (firstOne != null) {
			Double corrCoef = firstOne.getFirst();
			Double rSquare = firstOne.getSecond();

			Double intercept = secondOne.getFirst();
			Double slope = secondOne.getSecond();
			rList.add(corrCoef);
			rSquareList.add(rSquare);
			interceptList.add(intercept);
			slopeList.add(slope);
		}
		displayOneActResModel();
		rList.clear();
		rSquareList.clear();
		interceptList.clear();
		slopeList.clear();

	}

	public JTable getShiftTables() {
		return dTable1;

	}

	public JTable getShiftTables1() {
		return dTable2;

	}

	public JTable getShiftTables2() {
		return dTable3;

	}

	public JTable getShiftTables3() {
		return dTable4;

	}

	public List<Map<String, JPanel>> getDisplayPanel1() {
		return namePanelList1;
	}

	public JPanel getDisplayPanel2() {
		return jPanel2;
	}

	public List<Map<String, JPanel>> getDisplayPanel3() {
		return namePanelList;
	}

	public JPanel getDisplayPanel4() {
		return jPanel4;
	}

	public JPanel getDisplayPanel5() {
		return jPanel5;
	}

}
