package org.processmining.plugins.workloadperiod.main;

import java.util.ArrayList;
import java.util.Collections;
import java.util.Date;
import java.util.List;
import java.util.ListIterator;

import javax.swing.table.DefaultTableModel;
import javax.swing.table.TableModel;

import org.deckfour.xes.extension.std.XConceptExtension;
import org.deckfour.xes.extension.std.XLifecycleExtension;
import org.deckfour.xes.extension.std.XOrganizationalExtension;
import org.deckfour.xes.extension.std.XTimeExtension;
import org.deckfour.xes.model.XEvent;
import org.deckfour.xes.model.XLog;
import org.deckfour.xes.model.XTrace;
import org.processmining.framework.util.collection.ComparablePair;

/**
 * Check the log and see if any start events exist or not in the event log.
 * 
 * @author jnakatumba
 * 
 */

public class WLMainTable {

	private XLog log;
	private ComparablePair<String, Date> cidDate;
	private Date startDate;
	private Date compDate;

	private List<String> caseIdList;
	private ArrayList<ComparablePair<String, Date>> cidDateList;

	private List<String> actNames;
	private List<String> resNamesStart;

	private List<Date> compDatesList;
	private List<Date> startDatesList;
	private List<Long> waitingTimesList;
	private List<Long> serviceTimesList;

	private Long waitingTime = 0L;
	private Long prevResTime = 0L;
	private Long prevCasTime = 0L;

	private String resName;
	private String caseId;
	private String aName;
	private Date rDate;
	private int inI;

	private List<ComparablePair<String, Date>> resDateList;

	private ComparablePair<String, Date> resDate;
	/**
	 * Initialize the plugin by checking if any start events exist in the log
	 * 
	 * @param context
	 *            The pluginContext in which this plugin is executed.
	 * 
	 * @param log
	 *            The log that needs to be filtered.
	 * 
	 */

	public WLMainTable(XLog log) {
		this.log = log;

		actNames = new ArrayList<String>();
		compDatesList = new ArrayList<Date>();
		caseIdList = new ArrayList<String>();
		startDatesList = new ArrayList<Date>();
		waitingTimesList = new ArrayList<Long>();
		serviceTimesList = new ArrayList<Long>();

		resDateList = new ArrayList<ComparablePair<String, Date>>();
		resDate = new ComparablePair<String, Date>(null, null);
		resNamesStart = new ArrayList<String>();

		cidDate = new ComparablePair<String, Date>(null, null);
		cidDateList = new ArrayList<ComparablePair<String, Date>>();

		init();
		getGeneralInformation();
	}

	private void init() {
		for (XTrace trace : log) {
			for (XEvent event : trace) {
				String eventType = XLifecycleExtension.instance().extractTransition(event);
				if (eventType.equals("start")) {
					startDate = XTimeExtension.instance().extractTimestamp(event);
					caseId = XConceptExtension.instance().extractName(trace);
					aName = XConceptExtension.instance().extractName(event);

				} else if (eventType.equals("complete")) {

					String cid = XConceptExtension.instance().extractName(trace);
					String aNameC = XConceptExtension.instance().extractName(event);
					compDate = XTimeExtension.instance().extractTimestamp(event);
					resName = XOrganizationalExtension.instance().extractResource(event);
					if (caseId != null) {
						if (cid.equals(caseId) == true) {
							if (aName != null) {
								if (aName.equals(aNameC)) {
									cidDate = getDate(caseId, compDate);
									resDate = getDate(resName, compDate);
									cidDateList.add(cidDate);
									resDateList.add(resDate);
								}
							}
						}
					}
				}
			}
		}
		Collections.sort(resDateList);
	}
	
	private void getGeneralInformation() {
		for (XTrace trace : log) {
			for (XEvent event : trace) {
				String eventType = XLifecycleExtension.instance().extractTransition(event);
				if (eventType.equals("start")) {
					startDate = XTimeExtension.instance().extractTimestamp(event);
					caseId = XConceptExtension.instance().extractName(trace);
					aName = XConceptExtension.instance().extractName(event);
					resName = XOrganizationalExtension.instance().extractResource(event);

				} else if (eventType.equals("complete")) {

					String cid = XConceptExtension.instance().extractName(trace);
					String aNameC = XConceptExtension.instance().extractName(event);
					//resName = XOrganizationalExtension.instance().extractResource(event);
					compDate = XTimeExtension.instance().extractTimestamp(event);
					if (caseId != null) {
						if (cid.equals(caseId)) {
							if (aName != null) {
								if (aName.equals(aNameC)) {
									getResourceDetails();
									startDatesList.add(startDate);
									caseIdList.add(caseId);
									actNames.add(aName);
									compDatesList.add(compDate);
									waitingTimesList.add(waitingTime);
									resNamesStart.add(resName);

									long stTime = startDate.getTime();
									long compsTime = compDate.getTime();
									long serviceTimes = compsTime - stTime;
									serviceTimes = serviceTimes / 1000;
									serviceTimes = serviceTimes / 60;
									serviceTimesList.add(serviceTimes);
								}
							}

						}
					}
				}
			}
		}
	}

	/**
	 * Check to see if the current resource executed any event before
	 */
	private void getResourceDetails() {
		String rName;
		int inII = 0;
		now: for (ListIterator<ComparablePair<String, Date>> resDateNext = resDateList.listIterator(); resDateNext
				.hasNext();) {
			if (resDateNext.hasNext() == true) {
				ComparablePair<String, Date> currentPair = resDateNext.next();
				rName = currentPair.getFirst();
				rDate = currentPair.getSecond();
				//inII++;
				if (rName.equals(resName) == true) {
					if (rDate.compareTo(compDate) == 0) {
						int index = resDateNext.previousIndex();
						next: for (ListIterator<ComparablePair<String, Date>> resDatePrev = resDateList
								.listIterator(index); resDatePrev.hasPrevious();) {
							if (resDatePrev.hasPrevious() == true) {
								ComparablePair<String, Date> previousP = resDatePrev.previous();
								rName = previousP.getFirst();
								if (rName.equals(resName) == true) {
									rDate = previousP.getSecond();
									prevResTime = rDate.getTime();

									getServiceTimesOnCas();
									inII++;
									break next;
								} else {
									getServiceTimesOnCas();
									prevResTime = 0L;
									inII++;
									break now;
								}
							}
						}
						getServiceTimesOnCas();
						prevResTime = 0L;
						inII++;
						break now;
					}
				}
			}

		}


	}

	
	private void getServiceTimesOnCas() {
		int inII = 0;
		now: for (ListIterator<ComparablePair<String, Date>> caseDateNext = cidDateList.listIterator(inII); caseDateNext
				.hasNext();) {
			if (caseDateNext.hasNext() == true) {
				ComparablePair<String, Date> currentPair = caseDateNext.next();
				String cid = currentPair.getFirst();
				Date eDate = currentPair.getSecond();
				inII++;
				int index = caseDateNext.previousIndex();
				if (cid.equals(caseId) && eDate.equals(compDate)) {

					next: for (ListIterator<ComparablePair<String, Date>> caseDatePrev = cidDateList
							.listIterator(index); caseDatePrev.hasPrevious();) {

						if (caseDatePrev.hasPrevious() == true) {
							ComparablePair<String, Date> previousP = caseDatePrev.previous();
							String cidP = previousP.getFirst();
							Date dateP = previousP.getSecond();
							inI++;
							if (cidP.equals(cid)) {
								prevCasTime = dateP.getTime();
								waitingTime = Math.max(0, (prevResTime - prevCasTime));

								//check if first one for resource get it entirely on the case perspective and the other way round
								waitingTime = waitingTime / 1000;
								waitingTime = waitingTime / 60;

								break next;

							} else {

								prevCasTime = 0L;
								if (startDate.compareTo(rDate) < 0) {
									waitingTime = Math.max(0, (prevResTime - prevCasTime));
									waitingTime = waitingTime / 1000;
									waitingTime = waitingTime / 60;
									inI++;
									break now;
								} else {
									prevResTime = 0L;
									waitingTime = Math.max(0, (prevResTime - prevCasTime));
									waitingTime = waitingTime / 1000;
									waitingTime = waitingTime / 60;
									inI++;
									break now;
								}
							}
						}
					}

					break now;
				}
			}

		}

	}
	/**
	 * Checks the information based on the case perspective and then edits the
	 * event with the new information obtained.
	 */


	private ComparablePair<String, Date> getDate(String rName, Date timeStamp) {
		return new ComparablePair<String, Date>(rName, timeStamp);
	}

	public TableModel getMainTable() {

		DefaultTableModel mainModel = new DefaultTableModel();

		mainModel.addColumn("Case Id", caseIdList.toArray());
		mainModel.addColumn("Start Times", startDatesList.toArray());
		mainModel.addColumn("Resource Names", resNamesStart.toArray());
		mainModel.addColumn("Activity Names", actNames.toArray());
		mainModel.addColumn("Completion Times", compDatesList.toArray());
		mainModel.addColumn("Waiting Times", waitingTimesList.toArray());
		mainModel.addColumn("Service Times", serviceTimesList.toArray());

		return mainModel;
	}

	/**
	 * Returns the Mined Log
	 * 
	 * @return Log
	 */
	public XLog getLog() {
		return log;
	}

}
