package org.processmining.plugins.workloadperiod.main;

import java.util.ArrayList;
import java.util.Collection;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Set;
import java.util.TreeSet;

import org.deckfour.xes.classification.XEventClass;
import org.deckfour.xes.classification.XEventClasses;
import org.deckfour.xes.extension.std.XConceptExtension;
import org.deckfour.xes.extension.std.XLifecycleExtension;
import org.deckfour.xes.extension.std.XOrganizationalExtension;
import org.deckfour.xes.extension.std.XTimeExtension;
import org.deckfour.xes.info.XLogInfo;
import org.deckfour.xes.model.XAttributeMap;
import org.deckfour.xes.model.XEvent;
import org.deckfour.xes.model.XLog;
import org.deckfour.xes.model.XTrace;

/**
 * Workload Analysis Miner
 * 
 * Mines a log for different aspects of resource behavior.
 * 
 * @author jnakatumba
 * 
 */

public class WLMines {

	/**
	 * Get all traces from a log.
	 * 
	 * @param log
	 *            The log to get the traces from.
	 * @return All the traces from the log.
	 */
	static public Collection<XTrace> getTraces(XLog log) {
		return log;
	}

	/**
	 * Get all events from a log.
	 * 
	 * @param trace
	 *            The log to get the traces from.
	 * @return All the events from the log.
	 */
	static public Collection<XEvent> getEvents(XTrace trace) {
		return trace;
	}

	/**
	 * Get information about the timestamps from a given event.
	 * 
	 * @param event
	 *            The event to obtain the timestamp from.
	 * @return The timestamp from a given event.
	 */

	static public Date getTimeStamp(XEvent event) {
		Date d = XTimeExtension.instance().extractTimestamp(event);
		return d;

	}

	/**
	 * Get the information about the lifecycle transition of an event.
	 * 
	 * @param event
	 *            Event to get the information from.
	 * @return The lifecycle transition of the event.
	 */
	static public String getEventExtension(XEvent event) {
		String eventType;
		eventType = XLifecycleExtension.instance().extractTransition(event);
		return eventType;

	}

	/**
	 * Get the information about the originator who caused the event.
	 * 
	 * @param event
	 *            Event to get the information from.
	 * @return The name of the originator of the event.
	 */

	static public String getResourceName(XEvent event) {
		String resNames = XOrganizationalExtension.instance().extractResource(event);
		return resNames;
	}

	/**
	 * Get the information about the event type of an event.
	 * 
	 * @param event
	 *            Event to get the information from.
	 * @return The event type of the event.
	 */

	static public String getActivityName(XEvent event) {
		String aNames = XConceptExtension.instance().extractName(event);
		return aNames;
	}

	/**
	 * Get the data attributes of an event.
	 * 
	 * @param event
	 *            The event to get the data attribute from.
	 * @return The data attributes of the event.
	 */

	static public Map<String, String> getDataAttributes(XEvent event) {
		XAttributeMap attributeMap = event.getAttributes();
		HashMap<String, String> map = new HashMap<String, String>();
		for (String label : attributeMap.keySet()) {
			map.put(label, attributeMap.get(label).toString());
		}
		return map;
	}

	/**
	 * Collects all the information about the event type of an event.
	 * 
	 * @param Info
	 *            Info to get the information from.
	 * @return The set of all event types.
	 */
	static public Set<String> getActivity(XLogInfo info) {
		XEventClasses eventClasses = info.getEventClasses();
		TreeSet<String> set = new TreeSet<String>();
		for (XEventClass eventClass : eventClasses.getClasses()) {
			set.add(eventClass.getId());
		}
		return set;
	}

	/**
	 * Collects all the information about the event type of an event.
	 * 
	 * @param Info
	 *            Info to get the information from.
	 * @return The collection of all originators.
	 */
	static public Set<String> getResourceNames(XLogInfo info) {
		XEventClasses eventClasses = null;

		eventClasses = info.getResourceClasses();
		TreeSet<String> set = new TreeSet<String>();
		for (XEventClass eventClass : eventClasses.getClasses()) {
			set.add(eventClass.getId());
		}
		return set;
	}

	static public List<String> getResources(XLogInfo info) {
		List<String> originatorList = new ArrayList<String>();
		XEventClasses originators = info.getResourceClasses();

		originators.getClasses();
		originatorList = new ArrayList<String>();
		for (int k = 0; k < originators.getClasses().size(); k++) {
			originatorList.add(originators.getByIndex(k).toString());
		}
		return originatorList;
	}
}
