package org.processmining.plugins.workloadperiod.main;

import java.util.Collection;
import java.util.TreeSet;

import org.deckfour.xes.model.XLog;
import org.processmining.contexts.gui.GUIPluginContext;
import org.processmining.framework.plugin.PluginContext;

/**
 * Workload Analysis Input
 * 
 * The inputs needed for the plugin
 * 
 * @author jnakatumba
 * 
 */
public class WLSettings {

	private Collection<WLSimpleTypes> fDimension = new TreeSet<WLSimpleTypes>();
	private WLShiftTypes refShiftType = WLShiftTypes.COMPLETE;
	private WLShiftTypes loadShiftType = WLShiftTypes.COMPLETE;
	private Integer hoursShift = 6;
	private String wLType = "How Busy";
	private String regressionType = "Simple Regression";
	private String speedType = "Service Times";

	/**
	 * The Log to mine the results from.
	 */
	private XLog mylog;

	/**
	 * Gets the information from the Main Table
	 * 
	 * @param log
	 *            The Required log
	 */

	public WLSettings(PluginContext context, XLog log, Integer hours,
			WLShiftTypes refShiftTypes, WLShiftTypes loadShiftTypes,
			Collection<WLSimpleTypes> simpleOption, String displayChoice) {
		this.mylog = log;
		this.hoursShift = hours;
		this.refShiftType = refShiftTypes;
		this.loadShiftType = loadShiftTypes;
		this.regressionType = displayChoice;
		this.fDimension = simpleOption;
		refShiftType = WLShiftTypes.START;
		loadShiftType = WLShiftTypes.START;
	}

	public WLSettings(GUIPluginContext context, XLog log, Integer hours,
			WLShiftTypes refShiftTypes, WLShiftTypes loadShiftTypes,
			Collection<WLSimpleTypes> simpleOption, String displayChoice) {
		this.mylog = log;
		this.hoursShift = hours;
		this.refShiftType = refShiftTypes;
		this.loadShiftType = loadShiftTypes;
		this.regressionType = displayChoice;
		this.fDimension = simpleOption;

	}

	public WLSettings(Integer hours, WLShiftTypes refShiftTypes,
			WLShiftTypes loadShiftTypes,
			Collection<WLSimpleTypes> simpleOption, String displayChoice) {
		this.hoursShift = hours;
		this.refShiftType = refShiftTypes;
		this.loadShiftType = loadShiftTypes;
		this.regressionType = displayChoice;
		this.fDimension = simpleOption;

	}

	public WLSettings() {
		// TODO Auto-generated constructor stub
	}

	public WLShiftTypes getRefShiftTypes() {
		return refShiftType;
	}

	public WLShiftTypes getLoadShiftTypes() {
		return loadShiftType;
	}

	public WLShiftTypes setRefEventType(WLShiftTypes shiftTypes) {
		WLShiftTypes oShifts = refShiftType;
		refShiftType = shiftTypes;
		return oShifts;
	}

	public WLShiftTypes setLoadEventType(WLShiftTypes shiftTypes) {
		WLShiftTypes oShifts = loadShiftType;
		loadShiftType = shiftTypes;
		return oShifts;
	}

	public Collection<WLSimpleTypes> getSimpleDisplay() {
		return fDimension;

	}

	public Integer getHours() {
		return hoursShift;
	}

	public String getDisplayChoice() {
		return regressionType;
	}

	public String getWorkLoadType() {
		return wLType;
	}

	public String getSpeedType() {
		return speedType;
	}

	public Collection<WLSimpleTypes> setSimpleFilter(
			Collection<WLSimpleTypes> newFilter) {
		Collection<WLSimpleTypes> oldFilter = fDimension;
		fDimension = newFilter;
		return oldFilter;

	}

	public Integer setShiftHours(Integer hours) {
		Integer shPeriod = hoursShift;
		hoursShift = hours;
		return shPeriod;
	}

	public void getShiftHours(Integer hours) {
		hoursShift = hours;
	}

	public String setWorkLoadType(String wType) {
		String wlTypes = wLType;
		wLType = wType;
		return wlTypes;
	}

	public void getSpeedType(String sType) {
		speedType = sType;
	}

	public String setSpeedType(String sType) {
		String spType = speedType;
		speedType = sType;
		return spType;
	}

	public void getWorkLoadType(String rType) {
		wLType = rType;
	}

	public String setRegressionType(String rType) {
		String regType = regressionType;
		regressionType = rType;
		return regType;
	}

	public void getRegressionType(String rType) {
		regressionType = rType;
	}

	public XLog getLog() {
		return mylog;
	}

}
