package org.processmining.plugins.workloadperiod.main;

import java.util.ArrayList;
import java.util.Collection;
import java.util.Collections;
import java.util.Date;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.Set;
import java.util.TreeSet;

import javax.swing.JTable;
import javax.swing.table.DefaultTableModel;
import javax.swing.table.TableModel;

import org.processmining.framework.util.Pair;
import org.processmining.framework.util.collection.ComparablePair;
import org.processmining.plugins.workloadperiod.filter.WLFilter;
import org.processmining.plugins.workloadperiod.filter.WLIntervalsOne;
import org.processmining.plugins.workloadperiod.filter.WLTimeInterval;
import org.processmining.plugins.workloadperiod.regression.WLSimpleRegression;

/**
 * Workload Simple Class
 * 
 * @author jnakatumba
 * 
 */

public class WLSimple {

	private WLSimpleRegression myRegression;
	private WLIntervalsOne timeIntervals;
	private WLShiftTypes refshiftTypeList;
	private WLShiftTypes loadshiftTypeList;
	private Collection<WLSimpleTypes> fdimensionList;

	private Set<String> actNamesList;
	private Set<String> activityList;
	private Set<String> resourceList;

	private List<ComparablePair<Date, Long>> compDurationList;
	private List<ComparablePair<Date, Long>> startDurationList;
	private List<Date> compTimesList;
	private List<Date> startTimesList;

	private JTable mainTable;
	private JTable myTables;
	private JTable filterTable;
	private TableModel mainModel;
	private DefaultTableModel horModel = new DefaultTableModel();

	private String refshiftType;
	private String loadshiftType;
	private String activityName;
	private String resourceName;
	private String speedType;
	private StringBuffer myString = new StringBuffer();
	private Integer horizonPeriod;

	private Map<StringBuffer, Pair<Pair<Double, Double>, Pair<Double, Double>>> oneResActStart;
	private Map<String, Pair<Pair<Double, Double>, Pair<Double, Double>>> oneResAllActStart;
	private Map<String, Pair<Pair<Double, Double>, Pair<Double, Double>>> allResOneActStart;
	private Pair<Pair<Double, Double>, Pair<Double, Double>> allResActStart;
	private Pair<Pair<Double, Double>, Pair<Double, Double>> intermed;

	public WLSimple(WLSettings settings, WLMainTable mainAnalysis) {

		actNamesList = new TreeSet<String>();
		activityList = new TreeSet<String>();
		resourceList = new TreeSet<String>();

		fdimensionList = new TreeSet<WLSimpleTypes>();
		compTimesList = new ArrayList<Date>();

		compDurationList = new ArrayList<ComparablePair<Date, Long>>();
		startDurationList = new ArrayList<ComparablePair<Date, Long>>();
		startTimesList = new ArrayList<Date>();

		oneResActStart = new HashMap<StringBuffer, Pair<Pair<Double, Double>, Pair<Double, Double>>>();
		oneResAllActStart = new HashMap<String, Pair<Pair<Double, Double>, Pair<Double, Double>>>();
		allResActStart = new Pair<Pair<Double, Double>, Pair<Double, Double>>(null, null);
		allResOneActStart = new HashMap<String, Pair<Pair<Double, Double>, Pair<Double, Double>>>();

		mainModel = mainAnalysis.getMainTable();
		mainTable = new JTable(mainModel);

		horizonPeriod = settings.getHours();

		fdimensionList = settings.getSimpleDisplay();
		refshiftTypeList = settings.getRefShiftTypes();
		loadshiftTypeList = settings.getLoadShiftTypes();
		speedType = settings.getSpeedType();

		if (speedType == null || horizonPeriod == 0 || fdimensionList.isEmpty() == true) {
			speedType = "Service Times";
			horizonPeriod = 6;
			fdimensionList.add(WLSimpleTypes.ONERESOURCEONETASK);
		}

		getResourceNameSt();
		getActivityName();
		getRefShiftTypes();
		getLoadShiftTypes();
		getNewDimension();

	}

	private void getResourceNameSt() {
		String resNames;
		for (Integer colIndex = 0; colIndex < mainTable.getColumnCount(); colIndex++) {
			if (mainTable.getColumnName(colIndex).equals("Resource Names")) {
				for (Integer rowIndex = 0; rowIndex < mainTable.getRowCount(); rowIndex++) {
					Object value = mainTable.getValueAt(rowIndex, colIndex);
					if (value != null) {
						resNames = (String) value;
						resourceList.add(resNames);
					}
				}
			}

		}

	}

	/**
	 * Gets the activity Names
	 * 
	 * @return The Activity Names
	 */
	private void getActivityName() {
		for (Integer colIndex = 0; colIndex < mainTable.getColumnCount(); colIndex++) {
			if (mainTable.getColumnName(colIndex).equals("Activity Names")) {
				for (Integer rowIndex = 0; rowIndex < mainTable.getRowCount(); rowIndex++) {
					Object value = mainTable.getValueAt(rowIndex, colIndex);
					if (value != null) {
						String actNames = (String) value;
						activityList.add(actNames);
					}
				}
			}

		}

	}

	private void getRefShiftTypes() {
		if (refshiftTypeList.equals(WLShiftTypes.START)) {
			refshiftType = "Start";

		}
		if (refshiftTypeList.equals(WLShiftTypes.COMPLETE)) {
			refshiftType = "Complete";

		}
		if (refshiftTypeList.equals(WLShiftTypes.STARTCOMPLETE)) {
			refshiftType = "StartComplete";

		}

	}

	private void getLoadShiftTypes() {
		if (loadshiftTypeList.equals(WLShiftTypes.START)) {
			loadshiftType = "Start";

		}
		if (loadshiftTypeList.equals(WLShiftTypes.COMPLETE)) {
			loadshiftType = "Complete";

		}
		if (loadshiftTypeList.equals(WLShiftTypes.STARTCOMPLETE)) {
			loadshiftType = "StartComplete";

		}

	}

	private void getNewDimension() {
		if (fdimensionList.contains(WLSimpleTypes.ONERESOURCEONETASK)) {
			displayOneResAct();

		}
		if (fdimensionList.contains(WLSimpleTypes.ALLTASKSONERESOURCE)) {
			displayOneResAllAct();

		}

		if (fdimensionList.contains(WLSimpleTypes.ALLRESOURCESTASKS)) {
			displayAllResAct();

		}
		if (fdimensionList.contains(WLSimpleTypes.ALLRESOURCESONETASK)) {
			displayOneActAllRes();

		}

	}

	private void displayOneResAllAct() {
		Object[] reList = resourceList.toArray();
		for (int resIndex = 0; resIndex < resourceList.size(); resIndex++) {
			resourceName = (String) reList[resIndex];

			/**
			 * Filter the Main Table
			 */
			WLFilter myFilter = new WLFilter(resourceName, mainTable);
			myTables = new JTable();
			myTables = myFilter.getFilterTable();

			if (speedType.equals("Service Times")) {
				// obtain the service times
				WLSelectColumn selectCol = new WLSelectColumn(myTables);

				startTimesList = selectCol.getStartTimes();
				compTimesList = selectCol.getCompleteTimes();
				startDurationList = selectCol.getStartDurations();
				compDurationList = selectCol.getCompleteDurations();
			} else {
				// obtain the waiting times
				WLSelectColumnWT selectCol = new WLSelectColumnWT(myTables);

				startTimesList = selectCol.getStartTimes();
				compTimesList = selectCol.getCompleteTimes();
				startDurationList = selectCol.getStartDurations();
				compDurationList = selectCol.getCompleteDurations();
			}

			if (startTimesList.size() > 5) {
				Collections.sort(startTimesList);
				Collections.sort(compTimesList);
				Collections.sort(startDurationList);
				Collections.sort(compDurationList);

				double corrCoef = 0.0;
				double intercept = 0.0;
				double slope = 0.0;
				double rSquare = 0.0;

				timeIntervals = new WLIntervalsOne(new WLTimeInterval(horizonPeriod), startTimesList,
						startDurationList, compDurationList, compTimesList, refshiftType, loadshiftType);
				startTimesList.clear();
				startDurationList.clear();
				compDurationList.clear();
				compTimesList.clear();

				/**
				 * Carry Out Regression Analysis
				 */

				myRegression = new WLSimpleRegression(timeIntervals.getCases(), timeIntervals.getServiceTime());

				corrCoef = myRegression.getRegression();
				intercept = myRegression.getIntercept();
				slope = myRegression.getSlope();
				rSquare = myRegression.getRSquare();

				// if (corrCoef < 0 || corrCoef > 0) {
				Pair<Double, Double> theSquare = new Pair<Double, Double>(corrCoef, rSquare);
				Pair<Double, Double> theSlope = new Pair<Double, Double>(intercept, slope);

				intermed = new Pair<Pair<Double, Double>, Pair<Double, Double>>(theSquare, theSlope);

				oneResAllActStart.put(resourceName, intermed);

				StringBuffer myString = new StringBuffer();
				myString.append(resourceName);
				myString.append("\n");
				myString.append("+");
				myString.append("\n");
				myString.append("NoCases");

				StringBuffer myString1 = new StringBuffer();
				myString1.append(resourceName);
				myString1.append("\n");
				myString1.append("+");
				myString1.append("\n");
				myString1.append("ServiceTime");

				horModel.addColumn(myString, timeIntervals.getCases().toArray());
				horModel.addColumn(myString1, timeIntervals.getServiceTime().toArray());

			}
		}

	}

	private void displayOneResAct() {

		for (Iterator<String> resList = resourceList.iterator(); resList.hasNext();) {
			if (resList.hasNext() == true) {
				resourceName = resList.next();

				WLFilter myFilter = new WLFilter(resourceName, mainTable);
				myTables = new JTable();
				myTables = myFilter.getFilterTable();

				for (Integer colIndex = 0; colIndex < myTables.getColumnCount(); colIndex++) {
					if (myTables.getColumnName(colIndex).equals("Activity Names")) {
						for (Integer rowIndex = 0; rowIndex < myTables.getRowCount(); rowIndex++) {
							Object value = myTables.getValueAt(rowIndex, colIndex);
							if (value != null) {
								String actName = (String) value;
								actNamesList.add(actName);

							}
						}
					}

				}

				displayBasedonResFromActvity();
				actNamesList.clear();

			}
		}

	}

	private void displayBasedonResFromActvity() {

		getColumnsOut(myTables);

		Object[] reList = actNamesList.toArray();
		for (int resIndex = 0; resIndex < actNamesList.size(); resIndex++) {
			activityName = (String) reList[resIndex];

			WLFilter myFilter = new WLFilter(activityName, filterTable);
			JTable jtable = new JTable();
			jtable = myFilter.getFilterTable();

			if (speedType.equals("Service Times")) {
				WLSelectColumn selectCol = new WLSelectColumn(jtable);

				startTimesList = selectCol.getStartTimes();
				compTimesList = selectCol.getCompleteTimes();
				startDurationList = selectCol.getStartDurations();
				compDurationList = selectCol.getCompleteDurations();
			} else {
				WLSelectColumnWT selectCol = new WLSelectColumnWT(jtable);

				startTimesList = selectCol.getStartTimes();
				compTimesList = selectCol.getCompleteTimes();
				startDurationList = selectCol.getStartDurations();
				compDurationList = selectCol.getCompleteDurations();
			}

			Collections.sort(startTimesList);
			Collections.sort(compTimesList);
			Collections.sort(startDurationList);
			Collections.sort(compDurationList);
			if (startTimesList.size() > 5) {
				double corrCoef = 0.0;
				double intercept = 0.0;
				double rSquare = 0.0;
				double slope = 0.0;

				timeIntervals = new WLIntervalsOne(new WLTimeInterval(horizonPeriod), startTimesList,
						startDurationList, compDurationList, compTimesList, refshiftType, loadshiftType);
				startDurationList.clear();
				startTimesList.clear();
				compDurationList.clear();
				compTimesList.clear();

				myRegression = new WLSimpleRegression(timeIntervals.getCases(), timeIntervals.getServiceTime());
				corrCoef = myRegression.getRegression();
				intercept = myRegression.getIntercept();
				rSquare = myRegression.getRSquare();
				slope = myRegression.getSlope();

				// if (corrCoef < 0 || corrCoef > 0) {
				myString = new StringBuffer();
				myString.append(resourceName);
				myString.append("\n");
				myString.append("+");
				myString.append("\n");
				myString.append(activityName);

				Pair<Double, Double> theSquare = new Pair<Double, Double>(corrCoef, rSquare);
				Pair<Double, Double> theSlope = new Pair<Double, Double>(intercept, slope);

				intermed = new Pair<Pair<Double, Double>, Pair<Double, Double>>(theSquare, theSlope);
				oneResActStart.put(myString, intermed);

				StringBuffer myString = new StringBuffer();
				myString.append(resourceName);
				myString.append("\n");
				myString.append("+");
				myString.append("\n");
				myString.append("NoCases");

				StringBuffer myString1 = new StringBuffer();
				myString1.append(resourceName);
				myString1.append("\n");
				myString1.append("+");
				myString1.append("\n");
				myString1.append("ServiceTime");

				horModel.addColumn(myString, timeIntervals.getCases().toArray());
				horModel.addColumn(myString1, timeIntervals.getServiceTime().toArray());

				// }
			}

		}

	}

	private void displayAllResAct() {
		if (speedType.equals("Service Times")) {
			WLSelectColumn selectCol = new WLSelectColumn(mainTable);

			startTimesList = selectCol.getStartTimes();
			compTimesList = selectCol.getCompleteTimes();
			startDurationList = selectCol.getStartDurations();
			compDurationList = selectCol.getCompleteDurations();
		} else {
			WLSelectColumnWT selectCol = new WLSelectColumnWT(mainTable);

			startTimesList = selectCol.getStartTimes();
			compTimesList = selectCol.getCompleteTimes();
			startDurationList = selectCol.getStartDurations();
			compDurationList = selectCol.getCompleteDurations();
		}

		Collections.sort(startTimesList);
		Collections.sort(compTimesList);
		Collections.sort(startDurationList);
		Collections.sort(compDurationList);

		double corrCoef = 0.0;
		double intercept = 0.0;
		double slope = 0.0;
		double rSquare = 0.0;
		// if (startTimesList.size() > 5) {

		timeIntervals = new WLIntervalsOne(new WLTimeInterval(horizonPeriod), startTimesList, startDurationList,
				compDurationList, compTimesList, refshiftType, loadshiftType);
		startTimesList.clear();
		startDurationList.clear();
		compDurationList.clear();
		compTimesList.clear();

		/**
		 * Carry Out Regression Analysis
		 */

		myRegression = new WLSimpleRegression(timeIntervals.getCases(), timeIntervals.getServiceTime());

		corrCoef = myRegression.getRegression();
		intercept = myRegression.getIntercept();
		slope = myRegression.getSlope();
		rSquare = myRegression.getRSquare();

		// if (corrCoef < 0 || corrCoef > 0) {
		Pair<Double, Double> theSquare = new Pair<Double, Double>(corrCoef, rSquare);
		Pair<Double, Double> theSlope = new Pair<Double, Double>(intercept, slope);

		intermed = new Pair<Pair<Double, Double>, Pair<Double, Double>>(theSquare, theSlope);

		allResActStart = intermed;

		StringBuffer myString = new StringBuffer();
		myString.append("NoCases");

		StringBuffer myString1 = new StringBuffer();
		myString1.append("ServiceTime");

		horModel.addColumn(myString, timeIntervals.getCases().toArray());
		horModel.addColumn(myString1, timeIntervals.getServiceTime().toArray());
		// }
	}

	private void displayOneActAllRes() {
		Object[] actList = activityList.toArray();
		for (int actIndex = 0; actIndex < activityList.size(); actIndex++) {
			activityName = (String) actList[actIndex];

			/**
			 * Filter the Main Table
			 */

			WLFilter myFilter = new WLFilter(activityName, mainTable);
			JTable myTables = new JTable();
			myTables = myFilter.getFilterTable();

			/**
			 * Obtain the Intervals
			 */
			if (speedType.equals("Service Times")) {
				WLSelectColumn selectCol = new WLSelectColumn(myTables);

				startTimesList = selectCol.getStartTimes();
				compTimesList = selectCol.getCompleteTimes();
				startDurationList = selectCol.getStartDurations();
				compDurationList = selectCol.getCompleteDurations();
			} else {
				WLSelectColumnWT selectCol = new WLSelectColumnWT(myTables);

				startTimesList = selectCol.getStartTimes();
				compTimesList = selectCol.getCompleteTimes();
				startDurationList = selectCol.getStartDurations();
				compDurationList = selectCol.getCompleteDurations();
			}

			Collections.sort(startTimesList);
			Collections.sort(compTimesList);
			Collections.sort(startDurationList);
			Collections.sort(compDurationList);

			double corrCoef = 0.0;
			double intercept = 0.0;
			double rSquare = 0.0;
			double slope = 0.0;
			if (startTimesList.size() > 10) {
				timeIntervals = new WLIntervalsOne(new WLTimeInterval(horizonPeriod), startTimesList,
						startDurationList, compDurationList, compTimesList, refshiftType, loadshiftType);

				startTimesList.clear();
				compDurationList.clear();
				compTimesList.clear();
				startDurationList.clear();

				/**
				 * Carry out Correlation Analysis
				 */
				myRegression = new WLSimpleRegression(timeIntervals.getCases(), timeIntervals.getServiceTime());
				corrCoef = myRegression.getRegression();
				intercept = myRegression.getIntercept();
				rSquare = myRegression.getRSquare();
				slope = myRegression.getSlope();

				// if (corrCoef < 0 || corrCoef > 0) {

				Pair<Double, Double> theSquare = new Pair<Double, Double>(corrCoef, rSquare);
				Pair<Double, Double> theSlope = new Pair<Double, Double>(intercept, slope);

				intermed = new Pair<Pair<Double, Double>, Pair<Double, Double>>(theSquare, theSlope);

				allResOneActStart.put(activityName, intermed);

				StringBuffer myString = new StringBuffer();
				myString.append(activityName);
				myString.append("\n");
				myString.append("+");
				myString.append("\n");
				myString.append("NoCases");

				StringBuffer myString1 = new StringBuffer();
				myString1.append(activityName);
				myString1.append("\n");
				myString1.append("+");
				myString1.append("\n");
				myString1.append("ServiceTime");

				horModel.addColumn(myString, timeIntervals.getCases().toArray());
				horModel.addColumn(myString1, timeIntervals.getServiceTime().toArray());

				// }
			}
		}

	}

	/**
	 * Creates a new Table based on the filtered columns
	 */
	private void getColumnsOut(JTable table) {
		JTable filtTable = table;
		DefaultTableModel filterModel = new DefaultTableModel();
		/**
		 * Get the filtered contents form the General Table and then fill them
		 * into the Filter Table
		 */

		Object[] header = new Object[filtTable.getColumnCount()];
		for (Integer colIndex = 0; colIndex < filtTable.getColumnCount(); colIndex++) {
			header[colIndex] = filtTable.getColumnName(colIndex);
		}
		/**
		 * Replace the value in the filterModel with new values and header will
		 * hold the names of the new columns
		 */
		filterModel.setDataVector(null, header);

		/**
		 * Get the contents from the general table and pass them into the
		 * filterModel
		 */
		for (Integer rowIndex = 0; rowIndex < filtTable.getRowCount(); rowIndex++) {
			Object[] values = new Object[filtTable.getColumnCount()];
			for (Integer colIndex = 0; colIndex < filtTable.getColumnCount(); colIndex++) {
				Object value = filtTable.getValueAt(rowIndex, colIndex);
				values[colIndex] = value;
			}
			filterModel.insertRow(rowIndex, values);
		}

		filterTable = new JTable(filterModel);
	}

	/**
	 * public JTable getMainTables() { return jTable; }
	 */

	public Map<StringBuffer, Pair<Pair<Double, Double>, Pair<Double, Double>>> outputOneResActSt() {
		return oneResActStart;
	}

	public Map<String, Pair<Pair<Double, Double>, Pair<Double, Double>>> ouputOneResAllActSt() {
		return oneResAllActStart;
	}

	public Pair<Pair<Double, Double>, Pair<Double, Double>> outputAllResActStart() {
		return allResActStart;
	}

	public Map<String, Pair<Pair<Double, Double>, Pair<Double, Double>>> outputAllResOneActSt() {
		return allResOneActStart;
	}

}
