package org.processmining.plugins.workloadperiod.main;

import java.awt.Color;
import java.awt.Component;
import java.util.HashMap;
import java.util.Map;
import javax.swing.BorderFactory;
import javax.swing.BoxLayout;
import javax.swing.JComponent;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTabbedPane;
import javax.swing.JTable;
import javax.swing.JTextArea;
import javax.swing.SwingConstants;
import javax.swing.table.DefaultTableCellRenderer;
import javax.swing.table.TableColumn;

import org.processmining.contexts.uitopia.annotations.Visualizer;
import org.processmining.framework.plugin.PluginContext;
import org.processmining.framework.plugin.annotations.Plugin;

/**
 * Workload Analysis Output
 * 
 * The Tables to be displayed are populated from the Workload Analysis class
 * 
 * @author jnakatumba
 * 
 */

public class WLSimpleOutput {

	private Map<WLSimpleTypes, JTable> tables;
	private String dChoices;
	private String wlType;
	protected Color[] colors = null;
	private StringBuffer message;

	public WLSimpleOutput(PluginContext context) {
		tables = new HashMap<WLSimpleTypes, JTable>();
		message = new StringBuffer();

	}

	public void setDisplayMessage(StringBuffer messa) {
		message.append(messa);

	}

	public void setTables(WLSimpleTypes simpleTables, JTable table) {
		TableColumn tableColumns = table.getColumnModel().getColumn(1);
		DefaultTableCellRenderer caRenderer = new ColoredColumnRenderer1();

		caRenderer.setHorizontalAlignment(SwingConstants.RIGHT);
		tableColumns.setCellRenderer(caRenderer);

		TableColumn tableColumn = table.getColumnModel().getColumn(2);
		DefaultTableCellRenderer cRenderer = new ColoredColumnRenderer();

		cRenderer.setHorizontalAlignment(SwingConstants.RIGHT);
		tableColumn.setCellRenderer(cRenderer);

		TableColumn tableColumn2 = table.getColumnModel().getColumn(3);
		DefaultTableCellRenderer cRenderer2 = new ColoredColumnRenderer3();

		cRenderer2.setHorizontalAlignment(SwingConstants.RIGHT);
		tableColumn2.setCellRenderer(cRenderer2);

		TableColumn tableColumns1 = table.getColumnModel().getColumn(4);
		DefaultTableCellRenderer caRenderer1 = new ColoredColumnRenderer2();

		caRenderer.setHorizontalAlignment(SwingConstants.RIGHT);
		tableColumns1.setCellRenderer(caRenderer1);

		tables.put(simpleTables, table);
	}

	public void setTables1(WLSimpleTypes simpleTables, JTable table) {
		DefaultTableCellRenderer caRenderer = new ColoredColumnRenderer1();
		TableColumn tableColumn = table.getColumnModel().getColumn(1);
		DefaultTableCellRenderer cRenderer = new ColoredColumnRenderer();

		cRenderer.setHorizontalAlignment(SwingConstants.RIGHT);
		tableColumn.setCellRenderer(cRenderer);

		TableColumn tableColumn2 = table.getColumnModel().getColumn(2);
		DefaultTableCellRenderer cRenderer2 = new ColoredColumnRenderer3();

		cRenderer2.setHorizontalAlignment(SwingConstants.RIGHT);
		tableColumn2.setCellRenderer(cRenderer2);

		TableColumn tableColumns1 = table.getColumnModel().getColumn(3);
		DefaultTableCellRenderer caRenderer1 = new ColoredColumnRenderer2();

		caRenderer.setHorizontalAlignment(SwingConstants.RIGHT);
		tableColumns1.setCellRenderer(caRenderer1);

		tables.put(simpleTables, table);
	}

	public void setDChoice(String dChoice) {
		dChoices = dChoice;
	}

	public void setWLType(String wklMethod) {
		wlType = wklMethod;
	}

	@Plugin(name = "Visualize Resource Analysis", returnLabels = { "Visualized Resource Analysis" }, returnTypes = { JPanel.class }, parameterLabels = { "Resource Analysis" }, userAccessible = false)
	@Visualizer(name = "Workload visualiser")
	public static JComponent visualize(PluginContext context,
			WLSimpleOutput output) {
		JPanel panel = new JPanel();
		panel.setLayout(new BoxLayout(panel, BoxLayout.PAGE_AXIS));
		panel.setBorder(BorderFactory.createEmptyBorder(10, 10, 10, 10));
		JTabbedPane tabbedPane = new JTabbedPane();
		JScrollPane myPane = null;
		tabbedPane.setForeground(WLColorTypes.FOREGROUNDCOLOR.getColor());
		tabbedPane.setBackground(WLColorTypes.BACKGROUNDCOLOR.getColor());
		panel.add(tabbedPane);
		if (output.message.toString().isEmpty() == true) {

			if (output.wlType.equals("How Busy") == true) {
				if (output.dChoices.equals("Simple Regression")) {
					for (WLSimpleTypes shiftChoice : output.tables.keySet()) {
						myPane = new JScrollPane(output.tables.get(shiftChoice));
						tabbedPane.add(shiftChoice.getLabel(), myPane);

					}

				}
			} else {
				for (WLSimpleTypes shiftChoice : output.tables.keySet()) {
					myPane = new JScrollPane(output.tables.get(shiftChoice));
					tabbedPane.add(shiftChoice.getLabel(), myPane);

				}
			}
		} else {
			JTextArea textArea = new JTextArea();
			String displayMessage = output.message.toString();
			textArea.append(displayMessage);
			textArea.setWrapStyleWord(true);
			textArea.setColumns(20);
			textArea.setLineWrap(true);
			textArea.setRows(100);
			textArea.setEditable(false);
			myPane = new JScrollPane(textArea);
			panel.add(myPane);

		}
		return panel;
	}

	class ColoredColumnRenderer1 extends DefaultTableCellRenderer {

		private static final long serialVersionUID = 1L;

		public Component getTableCellRendererComponent(JTable table,
				Object value, boolean isSelected, boolean hasFocus,
				int rowIndex, int colIndex) {

			Component myComp = super.getTableCellRendererComponent(table,
					value, isSelected, hasFocus, rowIndex, colIndex);
			setHorizontalAlignment(SwingConstants.LEFT);

			if (((Double) value).doubleValue() < 1.1
					&& ((Double) value).doubleValue() > 0.7) {
				if (isSelected) {
					myComp.setBackground(new Color(
							(getColors(0).getRGB() ^ myComp.getBackground()
									.getRGB())));
					myComp.setForeground(new Color(Color.BLACK.getRGB()
							^ myComp.getForeground().getRGB()));
				} else {
					myComp.setBackground(getColors(0));
					myComp.setForeground(Color.BLACK);
				}
			} else if (((Double) value).doubleValue() < 0.7
					&& ((Double) value).doubleValue() > 0.3) {
				if (isSelected) {
					myComp.setBackground(new Color(
							(getColors(1).getRGB() ^ myComp.getBackground()
									.getRGB())));
					myComp.setForeground(new Color(Color.BLACK.getRGB()
							^ myComp.getForeground().getRGB()));
				} else {
					myComp.setBackground(getColors(1));
					myComp.setForeground(Color.BLACK);
				}
			} else if (((Double) value).doubleValue() < 0.3
					&& ((Double) value).doubleValue() > 0.0) {
				if (isSelected) {
					myComp.setBackground(new Color(
							(getColors(2).getRGB() ^ myComp.getBackground()
									.getRGB())));
					myComp.setForeground(new Color(Color.BLACK.getRGB()
							^ myComp.getForeground().getRGB()));
				} else {
					myComp.setBackground(getColors(2));
					myComp.setForeground(Color.BLACK);
				}
			} else if (((Double) value).doubleValue() < 0.0
					&& ((Double) value).doubleValue() > -0.3) {
				if (isSelected) {
					myComp.setBackground(new Color(
							(getColors(3).getRGB() ^ myComp.getBackground()
									.getRGB())));
					myComp.setForeground(new Color(Color.BLACK.getRGB()
							^ myComp.getForeground().getRGB()));
				} else {
					myComp.setBackground(getColors(3));
					myComp.setForeground(Color.BLACK);
				}
			} else if (((Double) value).doubleValue() < -0.3
					&& ((Double) value).doubleValue() > -0.7) {
				if (isSelected) {
					myComp.setBackground(new Color(
							(getColors(4).getRGB() ^ myComp.getBackground()
									.getRGB())));
					myComp.setForeground(new Color(Color.BLACK.getRGB()
							^ myComp.getForeground().getRGB()));
				} else {
					myComp.setBackground(getColors(4));
					myComp.setForeground(Color.BLACK);
				}
			} else if (((Double) value).doubleValue() < -0.7
					&& ((Double) value).doubleValue() > -1.1) {
				if (isSelected) {
					myComp.setBackground(new Color(
							(getColors(5).getRGB() ^ myComp.getBackground()
									.getRGB())));
					myComp.setForeground(new Color(Color.BLACK.getRGB()
							^ myComp.getForeground().getRGB()));
				} else {
					myComp.setBackground(getColors(5));
					myComp.setForeground(Color.BLACK);

				}

			} else {
				if (isSelected) {

					myComp.setBackground(Color.WHITE);
					myComp.setForeground(Color.BLACK);
				}
			}
			return myComp;
		}
	}

	class ColoredColumnRenderer extends DefaultTableCellRenderer {

		private static final long serialVersionUID = 1L;

		public Component getTableCellRendererComponent(JTable table,
				Object value, boolean isSelected, boolean hasFocus,
				int rowIndex, int colIndex) {

			Component myComp = super.getTableCellRendererComponent(table,
					value, isSelected, hasFocus, rowIndex, colIndex);
			setHorizontalAlignment(SwingConstants.LEFT);

			if (((Double) value).doubleValue() < 0.2) {
				if (isSelected) {
					myComp.setBackground(new Color(
							(getNewColors(2).getRGB() ^ myComp.getBackground()
									.getRGB())));
					myComp.setForeground(new Color(Color.BLACK.getRGB()
							^ myComp.getForeground().getRGB()));
				} else {
					myComp.setBackground(getNewColors(2));
					myComp.setForeground(Color.BLACK);
				}
			} else if (((Double) value).doubleValue() < 0.4
					&& ((Double) value).doubleValue() > 0.2) {
				if (isSelected) {
					myComp.setBackground(new Color(
							(getNewColors(1).getRGB() ^ myComp.getBackground()
									.getRGB())));
					myComp.setForeground(new Color(Color.BLACK.getRGB()
							^ myComp.getForeground().getRGB()));
				} else {
					myComp.setBackground(getNewColors(1));
					myComp.setForeground(Color.BLACK);
				}
			} else if (((Double) value).doubleValue() > 0.4) {
				if (isSelected) {
					myComp.setBackground(new Color(
							(getNewColors(0).getRGB() ^ myComp.getBackground()
									.getRGB())));
					myComp.setForeground(new Color(Color.BLACK.getRGB()
							^ myComp.getForeground().getRGB()));
				} else {
					myComp.setBackground(getNewColors(0));
					myComp.setForeground(Color.BLACK);
				}

			}
			return myComp;
		}
	}

	class ColoredColumnRenderer2 extends DefaultTableCellRenderer {

		private static final long serialVersionUID = 1L;

		public Component getTableCellRendererComponent(JTable table,
				Object value, boolean isSelected, boolean hasFocus,
				int rowIndex, int colIndex) {

			Component myComp = super.getTableCellRendererComponent(table,
					value, isSelected, hasFocus, rowIndex, colIndex);
			setHorizontalAlignment(SwingConstants.LEFT);

			myComp.setBackground(new Color((getNewColors(3).getRGB() ^ myComp
					.getBackground().getRGB())));
			myComp.setForeground(new Color(Color.BLACK.getRGB()
					^ myComp.getForeground().getRGB()));
			myComp.setBackground(getNewColors(3));
			myComp.setForeground(Color.BLACK);
			return myComp;

		}

	}

	class ColoredColumnRenderer3 extends DefaultTableCellRenderer {

		private static final long serialVersionUID = 1L;

		public Component getTableCellRendererComponent(JTable table,
				Object value, boolean isSelected, boolean hasFocus,
				int rowIndex, int colIndex) {

			Component myComp = super.getTableCellRendererComponent(table,
					value, isSelected, hasFocus, rowIndex, colIndex);
			setHorizontalAlignment(SwingConstants.LEFT);

			myComp.setBackground(new Color((getNewColors(4).getRGB() ^ myComp
					.getBackground().getRGB())));
			myComp.setForeground(new Color(Color.BLACK.getRGB()
					^ myComp.getForeground().getRGB()));
			myComp.setBackground(getNewColors(4));
			myComp.setForeground(Color.BLACK);
			return myComp;

		}

	}

	public Color getColors(int i) {
		colors = new Color[6];

		colors[0] = new Color(140, 81, 10); // 
		colors[1] = new Color(216, 179, 101); // 
		colors[2] = new Color(246, 232, 195); // 
		colors[3] = new Color(199, 234, 229); // 
		colors[4] = new Color(90, 180, 172); // 
		colors[5] = new Color(1, 102, 94); // 
		return colors[i];
	}

	/**
	 * Color Reference scheme from www.colorbrewer.org
	 * 
	 * @param Number
	 *            for the color
	 * @return The specified color
	 */

	public Color getNewColors(int i) {
		colors = new Color[5];

		colors[0] = new Color(35, 132, 67); // 
		colors[1] = new Color(120, 198, 121); // 
		colors[2] = new Color(194, 230, 153); // 
		colors[3] = new Color(251, 180, 185); // 
		colors[4] = new Color(254, 224, 182); // 

		return colors[i];
	}

}
