package org.processmining.plugins.workloadperiod.regression;

import java.util.List;
import java.util.ListIterator;

import org.apache.commons.math.stat.regression.SimpleRegression;

/**
 * Simple Regression analysis
 * 
 * Carries out Simple Regression analysis
 * 
 * @author jnakatumba
 * 
 */

public class WLSimpleRegression {
	/**
	 * Calls the SimpleRegression Class
	 */
	private SimpleRegression myRegression;

	/**
	 * The Array which will store the values to be analyzed
	 */
	private double[][] myArray;
	/*
	 * The List with the independent variables
	 */
	private List<Double> noCaseList;
	/**
	 * The List with the dependent variables
	 */
	private List<Double> serviceTimeList;
	/**
	 * Indices
	 */
	private int CI;
	private int cII = 0;
	private Double values;

	/**
	 * 
	 * @param noCasess
	 *            List with the independent variables
	 * @param serviceT
	 *            List with the dependent variables
	 */

	public WLSimpleRegression(List<Double> noCasess, List<Double> serviceT) {
		this.noCaseList = noCasess;
		this.serviceTimeList = serviceT;

		myRegression = new SimpleRegression();
		/**
		 * Gets the data from the lists into a two dimensional array
		 */
		getData();
		/**
		 * Passes the data to the regression method
		 */
		myRegression.addData(myArray);

	}

	/**
	 * Reads data from the array and passes it on into the Array
	 * 
	 * @return The two dimensional array
	 */
	public double[][] getData() {
		myArray = new double[noCaseList.size()][2];
		int listSizeCase = noCaseList.size();
		if (listSizeCase > 2) {
			for (int row = 0; row < listSizeCase; row++) {
				for (int col = 0; col < 2; col++) {
					if (col == 0) {
						getValues();

						myArray[row][col] = values;
						values = 0.0;
					} else {
						getValue();
						myArray[row][col] = values;
						values = 0.0;

					}
				}
			}
		}

		return myArray;
	}

	/**
	 * Returns values for the dependent variables
	 */
	@SuppressWarnings("unchecked")
	private void getValues() {

		for (ListIterator startRef = noCaseList.listIterator(cII); startRef.hasNext();) {
			if (startRef.hasNext() == true) {
				CI = startRef.nextIndex();
				values = (Double) startRef.next();
				cII++;
				break;

			}
		}
	}

	/**
	 * Returns values for the independent variables
	 */
	@SuppressWarnings("unchecked")
	private void getValue() {
		for (ListIterator compRef = serviceTimeList.listIterator(CI); compRef.hasNext();) {
			if (compRef.hasNext() == true) {
				values = (Double) compRef.next();
				break;

			}
		}
	}

	/**
	 * Gets the Pearson product-moment correlation coefficient
	 * 
	 * @return The Correlation Coefficient
	 */
	public double getRegression() {

		return myRegression.getR();

	}

	/**
	 * Gets the Intercept of the Regression Line
	 * 
	 * @return The Intercept
	 */

	public double getIntercept() {

		return myRegression.getIntercept();
	}

	/**
	 * Returns the square of the Correlation Coefficient
	 * 
	 * @return The R-Square
	 */

	public double getRSquare() {

		return myRegression.getRSquare();
	}

	public double getSlope() {
		return myRegression.getSlope();
	}

}
