package org.processmining.plugins.workloadqueue;

import java.util.ArrayList;
import java.util.Collection;
import java.util.Collections;
import java.util.Date;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.Set;
import java.util.TreeSet;

import javax.swing.JTable;
import javax.swing.table.DefaultTableModel;
import javax.swing.table.TableModel;

import org.processmining.framework.util.Pair;
import org.processmining.framework.util.collection.ComparablePair;
import org.processmining.plugins.workloadperiod.filter.WLFilter;
import org.processmining.plugins.workloadperiod.main.WLMainTable;
import org.processmining.plugins.workloadperiod.main.WLSettings;
import org.processmining.plugins.workloadperiod.main.WLSimpleTypes;

/**
 * 
 * Workload Main Class Based on Queue Length
 * 
 * @author jnakatumba
 * 
 */

public class WQMain {

	private WQRegression qRegression;

	private Collection<WLSimpleTypes> fdimensionList;

	private String resName, actName;
	private Date cDate, fDate, sDate;
	private Integer y = 1;

	private Map<String, Pair<Pair<Double, Double>, Pair<Double, Double>>> oneResAllActStart;
	private Map<StringBuffer, Pair<Pair<Double, Double>, Pair<Double, Double>>> oneResOneActStart;
	private Map<String, Pair<Pair<Double, Double>, Pair<Double, Double>>> allResOneActStart;
	private Map<Integer, List<Long>> serviceTimeList;
	private Pair<Pair<Double, Double>, Pair<Double, Double>> intermed;
	private List<ComparablePair<Date, Long>> startDurationList;
	private ComparablePair<Date, Long> compDuration;

	private Set<String> reNamesList;
	private Set<String> actNames;
	private Set<String> actNamesList;

	private List<Long> stList;

	private TableModel mainModel;
	private JTable jTable;
	private JTable filterTable;
	private String speedType;

	public WQMain(WLSettings settings, WLMainTable mainAnalysis) {

		actNamesList = new TreeSet<String>();
		fdimensionList = new TreeSet<WLSimpleTypes>();
		reNamesList = new TreeSet<String>();
		actNames = new TreeSet<String>();

		oneResAllActStart = new HashMap<String, Pair<Pair<Double, Double>, Pair<Double, Double>>>();
		oneResOneActStart = new HashMap<StringBuffer, Pair<Pair<Double, Double>, Pair<Double, Double>>>();

		allResOneActStart = new HashMap<String, Pair<Pair<Double, Double>, Pair<Double, Double>>>();
		startDurationList = new ArrayList<ComparablePair<Date, Long>>();
		fdimensionList = settings.getSimpleDisplay();
		compDuration = new ComparablePair<Date, Long>(null, null);

		stList = new ArrayList<Long>();
		serviceTimeList = new HashMap<Integer, List<Long>>();
		mainModel = mainAnalysis.getMainTable();
		jTable = new JTable(mainModel);
		filterTable = new JTable();

		speedType = settings.getSpeedType();
		//get the resource and activity names in the log
		init();

		//get details based on the different filter dimensions
		getFilterDimension();

	}

	/**
	 * Obtain the resource and Activity Names
	 */

	private void init() {
		for (Integer colIndex = 0; colIndex < jTable.getColumnCount(); colIndex++) {
			if (jTable.getColumnName(colIndex).equals("Resource Names")) {
				for (Integer rowIndex = 0; rowIndex < jTable.getRowCount(); rowIndex++) {
					Object value = jTable.getValueAt(rowIndex, colIndex);
					if (value != null) {
						resName = (String) value;
						reNamesList.add(resName);

					}
				}
			}

		}
		for (Integer colIndex = 0; colIndex < jTable.getColumnCount(); colIndex++) {
			if (jTable.getColumnName(colIndex).equals("Activity Names")) {
				for (Integer rowIndex = 0; rowIndex < jTable.getRowCount(); rowIndex++) {
					Object value = jTable.getValueAt(rowIndex, colIndex);
					if (value != null) {
						String actName = (String) value;
						actNames.add(actName);

					}
				}
			}

		}

	}

	/**
	 * Get the start Times and the corresponding service times from the main
	 * Table
	 * 
	 * @param mainTable
	 *            Table obtained from the log
	 */
	private void selectRowsService(JTable mainTable) {
		startDurationList = new ArrayList<ComparablePair<Date, Long>>();

		for (Integer colIndex = 0; colIndex < mainTable.getColumnCount(); colIndex++) {
			if (mainTable.getColumnName(colIndex).equals("Start Times")) {
				for (Integer rowIndex = 0; rowIndex < mainTable.getRowCount(); rowIndex++) {
					Object values = mainTable.getValueAt(rowIndex, colIndex);
					if (values != null) {
						Date cDate = (Date) values;

						now: for (Integer colIndex5 = 0; colIndex5 < mainTable.getColumnCount(); colIndex5++) {
							if (mainTable.getColumnName(colIndex5).equals("Service Times")) {
								for (Integer rowIndex5 = 0; rowIndex5 < mainTable.getRowCount(); rowIndex5++) {
									Object valueA = mainTable.getValueAt(rowIndex, colIndex5);
									Long duration = (Long) valueA;
									if (valueA != null) {
										compDuration = getCompDuration(cDate, duration);
										startDurationList.add(compDuration);
										break now;
									}
								}
							}
						}
					}
				}
			}
		}

	}

	/**
	 * Get the start Times and the corresponding waiitng times from the main
	 * Table
	 * 
	 * @param mainTable
	 *            Table obtained from the log
	 */

	private void selectRowsWaiting(JTable mainTable) {
		startDurationList = new ArrayList<ComparablePair<Date, Long>>();

		for (Integer colIndex = 0; colIndex < mainTable.getColumnCount(); colIndex++) {
			if (mainTable.getColumnName(colIndex).equals("Start Times")) {
				for (Integer rowIndex = 0; rowIndex < mainTable.getRowCount(); rowIndex++) {
					Object values = mainTable.getValueAt(rowIndex, colIndex);
					if (values != null) {
						Date cDate = (Date) values;

						now: for (Integer colIndex5 = 0; colIndex5 < mainTable.getColumnCount(); colIndex5++) {
							if (mainTable.getColumnName(colIndex5).equals("Waiting Times")) {
								for (Integer rowIndex5 = 0; rowIndex5 < mainTable.getRowCount(); rowIndex5++) {
									Object valueA = mainTable.getValueAt(rowIndex, colIndex5);
									Long duration = (Long) valueA;
									if (valueA != null) {
										compDuration = getCompDuration(cDate, duration);
										startDurationList.add(compDuration);
										break now;
									}
								}
							}
						}
					}
				}
			}
		}

	}

	/**
	 * Get the display dimensions
	 */
	private void getFilterDimension() {

		if (fdimensionList.contains(WLSimpleTypes.ALLTASKSONERESOURCE)) {
			displayOneResAllAct();
		}

		if (fdimensionList.contains(WLSimpleTypes.ALLRESOURCESTASKS)) {
			displayAllResAllAct();
		}
		if (fdimensionList.contains(WLSimpleTypes.ALLRESOURCESONETASK)) {
			displayAllResOneAct();
		}

		if (fdimensionList.contains(WLSimpleTypes.ONERESOURCEONETASK)) {
			displayOneResOneAct();
		}
	}

	private void displayOneResOneAct() {
		for (Iterator<String> resList = reNamesList.iterator(); resList.hasNext();) {
			if (resList.hasNext() == true) {
				resName = resList.next();

				WLFilter myFilter = new WLFilter(resName, jTable);
				JTable myTables = myFilter.getFilterTable();

				for (Integer colIndex = 0; colIndex < myTables.getColumnCount(); colIndex++) {
					if (myTables.getColumnName(colIndex).equals("Activity Names")) {
						for (Integer rowIndex = 0; rowIndex < myTables.getRowCount(); rowIndex++) {
							Object value = myTables.getValueAt(rowIndex, colIndex);
							if (value != null) {
								String actName = (String) value;
								actNamesList.add(actName);

							}
						}
					}

				}

				displayBasedonResFromActvity(myTables);
				actNamesList.clear();

			}
		}

	}

	private void displayBasedonResFromActvity(JTable jTables) {
		List<Long> sTimes = new ArrayList<Long>();
		getColumnsOut(jTables);

		Object[] actList = actNamesList.toArray();
		for (int actIndex = 0; actIndex < actNamesList.size(); actIndex++) {
			actName = (String) actList[actIndex];

			WLFilter myFilter = new WLFilter(actName, filterTable);
			JTable jtable = myFilter.getFilterTable();
			//check if processing time is denoted by service time
			if (speedType.equals("Service Times")) {
				selectRowsService(jtable);
			} else {
				//if processing time is denoted by the waiting time
				selectRowsWaiting(jtable);
			}

			Collections.sort(startDurationList);
			if (startDurationList.size() > 4) {

				for (int i = 0; i < startDurationList.size(); i++) {
					ComparablePair<Date, Long> stDurationPair = startDurationList.get(i);
					cDate = stDurationPair.getFirst();
					Long stTimes = stDurationPair.getSecond();

					/**
					 * Calendar myCalendar = Calendar.getInstance();
					 * myCalendar.setTime(fDate); myCalendar.add(Calendar.HOUR,
					 * -1); cDate = myCalendar.getTime();
					 */

					getCasesAct(jTable);

					serviceTimeList.put(y, stList);
					sTimes.add(stTimes);

					y++;
					stList = new ArrayList<Long>();

				}

				qRegression = new WQRegression(serviceTimeList, sTimes);
				intermed = qRegression.getRegressionDetails();
				if (intermed != null) {
					StringBuffer myString = new StringBuffer();
					myString.append(resName);
					myString.append("\n");
					myString.append("+");
					myString.append("\n");
					myString.append(actName);

					oneResOneActStart.put(myString, intermed);
				}
				serviceTimeList = new HashMap<Integer, List<Long>>();
				sTimes = new ArrayList<Long>();
				startDurationList.clear();
				y = 0;

			}
		}

	}

	/**
	 * Based on one Resource and all the activities carried out
	 */
	private void displayOneResAllAct() {
		Object[] reList = reNamesList.toArray();
		List<Long> sTimes = new ArrayList<Long>();

		for (int resIndex = 0; resIndex < reNamesList.size(); resIndex++) {
			resName = (String) reList[resIndex];

			WLFilter myFilter = new WLFilter(resName, jTable);
			JTable myTables = myFilter.getFilterTable();

			if (speedType.equals("Service Times")) {
				selectRowsService(myTables);
			} else {
				selectRowsWaiting(myTables);
			}

			Collections.sort(startDurationList);
			if (startDurationList.size() > 4) {

				for (int i = 0; i < startDurationList.size(); i++) {
					ComparablePair<Date, Long> stDurationPair = startDurationList.get(i);
					cDate = stDurationPair.getFirst();
					Long stTimes = stDurationPair.getSecond();

					/**
					 * Calendar myCalendar = Calendar.getInstance();
					 * myCalendar.setTime(fDate); myCalendar.add(Calendar.HOUR,
					 * -1); cDate = myCalendar.getTime();
					 */

					getCasesRes(jTable);

					serviceTimeList.put(y, stList);
					sTimes.add(stTimes);

					y++;
					stList = new ArrayList<Long>();

				}

				qRegression = new WQRegression(serviceTimeList, sTimes);
				intermed = qRegression.getRegressionDetails();
				if (intermed != null) {
					oneResAllActStart.put(resName, intermed);
				}
				serviceTimeList = new HashMap<Integer, List<Long>>();
				sTimes = new ArrayList<Long>();
				startDurationList.clear();
				y = 0;

			}
		}

	}

	/**
	 * Based on one activity and all the resources that executed this activity
	 */
	private void displayAllResOneAct() {
		List<Long> sTimes = new ArrayList<Long>();

		Object[] actList = actNames.toArray();
		for (int actIndex = 0; actIndex < actNames.size(); actIndex++) {
			actName = (String) actList[actIndex];

			WLFilter myFilter = new WLFilter(actName, jTable);
			JTable myTables = myFilter.getFilterTable();
			//check if processing times are denoted by service times
			if (speedType.equals("Service Times")) {
				selectRowsService(myTables);
			} else {
				//check if processing times are denoted by waiting time
				selectRowsWaiting(myTables);
			}

			Collections.sort(startDurationList);
			if (startDurationList.size() > 10) {

				for (int i = 0; i < startDurationList.size(); i++) {
					ComparablePair<Date, Long> stDurationPair = startDurationList.get(i);
					cDate = stDurationPair.getFirst();
					Long stTimes = stDurationPair.getSecond();

					/**
					 * Calendar myCalendar = Calendar.getInstance();
					 * myCalendar.setTime(fDate); myCalendar.add(Calendar.HOUR,
					 * -1); cDate = myCalendar.getTime();
					 */

					getCasesAct(jTable);

					serviceTimeList.put(y, stList);
					sTimes.add(stTimes);

					y++;
					stList = new ArrayList<Long>();

				}

				qRegression = new WQRegression(serviceTimeList, sTimes);
				intermed = qRegression.getRegressionDetails();
				if (intermed != null) {
					allResOneActStart.put(actName, intermed);

				}
				serviceTimeList = new HashMap<Integer, List<Long>>();
				sTimes = new ArrayList<Long>();
				startDurationList.clear();
				y = 0;

			}
		}

	}

	/**
	 * Get the cases waiting based on the Main Table based on activity
	 * perspective
	 * 
	 * @param mainTable
	 *            table obtained from the log
	 */
	private void getCasesAct(JTable mainTable) {
		for (Integer colIndex = 0; colIndex < mainTable.getColumnCount(); colIndex++) {
			if (mainTable.getColumnName(colIndex).equals("Start Times")) {
				for (Integer rowIndex = 0; rowIndex < mainTable.getRowCount(); rowIndex++) {
					Object value = mainTable.getValueAt(rowIndex, colIndex);
					if (rowIndex != 0) {
						String cid = getCaseIds(rowIndex, mainTable);
						String cid1 = getCaseIds(rowIndex - 1, mainTable);
						if (cid != null && cid1 != null) {
							if (cid.equals(cid1) == true) {
								sDate = (Date) value;
								//if (sDate.compareTo(mDate) != 0) {
								fDate = getCompleteTime(rowIndex - 1, mainTable);

								if (fDate.compareTo(cDate) < 0 && sDate.compareTo(cDate) > 0) {
									String acName = getActivityNames(rowIndex, mainTable);
									if (acName.equals(actName)) {
										Long serviceTimes = getServiceTime(rowIndex, mainTable);
										stList.add(serviceTimes);

									}

								}
								//}
							}
						}
					}
				}
			}
		}

	}

	/**
	 * Get the cases waiting based on the Main Table based on resource
	 * perspective
	 * 
	 * @param mainTable
	 *            table obtained from the log
	 */
	private void getCasesRes(JTable mainTable) {
		for (Integer colIndex = 0; colIndex < mainTable.getColumnCount(); colIndex++) {
			if (mainTable.getColumnName(colIndex).equals("Start Times")) {
				for (Integer rowIndex = 0; rowIndex < mainTable.getRowCount(); rowIndex++) {
					Object value = mainTable.getValueAt(rowIndex, colIndex);
					if (rowIndex != 0) {
						String cid = getCaseIds(rowIndex, mainTable);
						String cid1 = getCaseIds(rowIndex - 1, mainTable);
						if (cid != null && cid1 != null) {
							if (cid.equals(cid1) == true) {
								sDate = (Date) value;
								fDate = getCompleteTime(rowIndex - 1, mainTable);
								//if (sDate.compareTo(mDate) != 0) {
								if (fDate.compareTo(cDate) < 0 && sDate.compareTo(cDate) > 0) {
									String rName = getResourceNames(rowIndex, mainTable);
									if (rName.equals(resName)) {
										Long serviceTimes = getServiceTime(rowIndex, mainTable);
										stList.add(serviceTimes);

									}

									//}
								}
							}
						}
					}
				}
			}
		}
	}

	/**
	 * Display based on all the activities and resources
	 */

	private void displayAllResAllAct() {
		//check if processing time is denoted by service time
		if (speedType.equals("Service Times")) {
			selectRowsService(jTable);
		} else {
			//check if processing time is denoted by waiting time
			selectRowsWaiting(jTable);
		}

		Collections.sort(startDurationList);
		List<Long> sTimes = new ArrayList<Long>();

		for (int i = 0; i < startDurationList.size(); i++) {
			ComparablePair<Date, Long> stDurationPair = startDurationList.get(i);
			cDate = stDurationPair.getFirst();
			Long stTimes = stDurationPair.getSecond();

			/**
			 * Calendar myCalendar = Calendar.getInstance();
			 * myCalendar.setTime(fDate); myCalendar.add(Calendar.HOUR, -3);
			 * cDate = myCalendar.getTime();
			 */

			getCases(jTable);
			serviceTimeList.put(y, stList);
			sTimes.add(stTimes);

			y++;
			stList = new ArrayList<Long>();
			

		}

		qRegression = new WQRegression(serviceTimeList, sTimes);
		intermed = qRegression.getRegressionDetails();

		serviceTimeList = new HashMap<Integer, List<Long>>();
		sTimes = new ArrayList<Long>();
		startDurationList.clear();
		y = 0;
	}

	/**
	 * Obtain cased based on all the tasks and resources
	 * 
	 * @param mainTable
	 *            Table mined from the log
	 */
	private void getCases(JTable mainTable) {
		for (Integer colIndex = 0; colIndex < mainTable.getColumnCount(); colIndex++) {
			if (mainTable.getColumnName(colIndex).equals("Start Times")) {
				for (Integer rowIndex = 0; rowIndex < mainTable.getRowCount(); rowIndex++) {
					Object value = mainTable.getValueAt(rowIndex, colIndex);
					if (rowIndex != 0) {
						String cid = getCaseIds(rowIndex, mainTable);
						String cid1 = getCaseIds(rowIndex - 1, mainTable);
						if (cid != null && cid1 != null) {
							if (cid.equals(cid1) == true) {
								sDate = (Date) value;
								fDate = getCompleteTime(rowIndex - 1, mainTable);

								if (fDate.compareTo(cDate) < 0 && sDate.compareTo(cDate) > 0) {
									Long serviceTimes = getServiceTime(rowIndex, mainTable);
									stList.add(serviceTimes);

								}

							}
						}
					}
				}
			}
		}

	}

	/**
	 * Obtain the Case Id based on a particular rowIndex
	 * 
	 * @param rowIndex
	 *            row Index used
	 * @param mainTable
	 *            Table mined from the log
	 * @return Case Id
	 */
	private String getCaseIds(int rowIndex, JTable mainTable) {
		String cid = null;
		now: for (Integer colIndex = 0; colIndex < mainTable.getColumnCount(); colIndex++) {
			if (mainTable.getColumnName(colIndex).equals("Case Id")) {
				Object value = mainTable.getValueAt(rowIndex, colIndex);
				if (value != null) {
					cid = (String) value;
					break now;
				}
			}
		}
		return cid;
	}

	/**
	 * Obtain the resource name based on a particular rowIndex
	 * 
	 * @param rowIndex
	 *            row Index used
	 * @param mainTable
	 *            Table mined from the log
	 * @return Resource Name
	 */
	private String getResourceNames(int rowIndex, JTable mainTable) {
		String rName = null;
		now: for (Integer colIndex = 0; colIndex < mainTable.getColumnCount(); colIndex++) {
			if (mainTable.getColumnName(colIndex).equals("Resource Names")) {
				Object value = mainTable.getValueAt(rowIndex, colIndex);
				if (value != null) {
					rName = (String) value;
					break now;
				}
			}
		}
		return rName;
	}

	/**
	 * Obtain the activity name based on a particular rowIndex
	 * 
	 * @param rowIndex
	 *            row Index used
	 * @param mainTable
	 *            Table mined from the log
	 * @return Activity Name
	 */
	private String getActivityNames(int rowIndex, JTable mainTable) {
		String actName = null;
		now: for (Integer colIndex = 0; colIndex < mainTable.getColumnCount(); colIndex++) {
			if (mainTable.getColumnName(colIndex).equals("Activity Names")) {
				Object value = mainTable.getValueAt(rowIndex, colIndex);
				if (value != null) {
					actName = (String) value;
					break now;
				}
			}
		}
		return actName;
	}

	/**
	 * Obtain the service time based on a particular rowIndex
	 * 
	 * @param rowIndex
	 *            row Index used
	 * @param mainTable
	 *            Table mined from the log
	 * @return Service Time
	 */
	private Long getServiceTime(int rowIndex, JTable mainTable) {
		Long sTimes = 0L;
		now: for (Integer colIndex = 0; colIndex < mainTable.getColumnCount(); colIndex++) {
			if (mainTable.getColumnName(colIndex).equals("Service Times")) {
				Object value = mainTable.getValueAt(rowIndex, colIndex);
				if (value != null) {
					sTimes = (Long) value;
					break now;

				}
			}
		}
		return sTimes;

	}

	/**
	 * Obtain the Completion Dates based on a particular rowIndex
	 * 
	 * @param rowIndex
	 *            row Index used
	 * @param mainTable
	 *            Table mined from the log
	 * @return Completion Dates
	 */
	private Date getCompleteTime(int rowIndex, JTable mainTable) {
		Date cTimes = null;
		now: for (Integer colIndex = 0; colIndex < mainTable.getColumnCount(); colIndex++) {
			if (mainTable.getColumnName(colIndex).equals("Completion Times")) {
				Object value = mainTable.getValueAt(rowIndex, colIndex);
				if (value != null) {
					cTimes = (Date) value;
					break now;

				}
			}
		}
		return cTimes;

	}

	private void getColumnsOut(JTable table) {
		JTable filtTable = table;
		DefaultTableModel filterModel = new DefaultTableModel();
		/**
		 * Get the filtered contents form the General Table and then fill them
		 * into the Filter Table
		 */

		Object[] header = new Object[filtTable.getColumnCount()];
		for (Integer colIndex = 0; colIndex < filtTable.getColumnCount(); colIndex++) {
			header[colIndex] = filtTable.getColumnName(colIndex);
		}
		/**
		 * Replace the value in the filterModel with new values and header will
		 * hold the names of the new columns
		 */
		filterModel.setDataVector(null, header);

		/**
		 * Get the contents from the general table and pass them into the
		 * filterModel
		 */
		for (Integer rowIndex = 0; rowIndex < filtTable.getRowCount(); rowIndex++) {
			Object[] values = new Object[filtTable.getColumnCount()];
			for (Integer colIndex = 0; colIndex < filtTable.getColumnCount(); colIndex++) {
				Object value = filtTable.getValueAt(rowIndex, colIndex);
				values[colIndex] = value;
			}
			filterModel.insertRow(rowIndex, values);
		}

		filterTable = new JTable(filterModel);
	}

	public Map<String, Pair<Pair<Double, Double>, Pair<Double, Double>>> ouputOneResAllActSt() {
		return oneResAllActStart;
	}

	public Map<String, Pair<Pair<Double, Double>, Pair<Double, Double>>> ouputAllResOneActSt() {

		return allResOneActStart;
	}

	public Map<StringBuffer, Pair<Pair<Double, Double>, Pair<Double, Double>>> ouputOneResOneActSt() {
		return oneResOneActStart;
	}

	public Pair<Pair<Double, Double>, Pair<Double, Double>> ouputAllResAllActSt() {
		return intermed;
	}

	private ComparablePair<Date, Long> getCompDuration(Date compDates, Long duration) {
		return new ComparablePair<Date, Long>(compDates, duration);
	}
}
