package org.processmining.plugins.workloadqueue;

import java.util.ArrayList;
import java.util.Date;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.Set;

import org.processmining.framework.util.Pair;
import org.processmining.plugins.workloadperiod.regression.WLSimpleRegression;

/**
 * Workload Queue Regression class
 * 
 * @author jnakatumba
 * 
 */

public class WQRegression {
	private Map<Integer, List<Long>> serviceTimeList;

	private WLSimpleRegression myRegression;
	private Pair<Pair<Double, Double>, Pair<Double, Double>> intermed;
	private List<Double> serList;
	private List<Double> noCasesList;
	List<Long> sTs;

	public WQRegression(Map<Integer, List<Long>> serviceTimeLists, List<Long> st) {
		this.serviceTimeList = serviceTimeLists;

		this.sTs = st;

		serList = new ArrayList<Double>();
		noCasesList = new ArrayList<Double>();
		new ArrayList<Date>();

		double corrCoef = 0.0;
		double intercept = 0.0;
		double slope = 0.0;
		double rSquare = 0.0;
		int noShift = 0;
		Set<Map.Entry<Integer, List<Long>>> stEntryList = serviceTimeList.entrySet();
		Iterator<Map.Entry<Integer, List<Long>>> iter = stEntryList.iterator();

		while (iter.hasNext()) {
			List<Long> serviceTime = iter.next().getValue();
			noShift = serviceTime.size();
			Double noShifts = Double.valueOf(noShift);
			noCasesList.add(noShifts);

		}

		for (int i = 0; i < sTs.size(); i++) {
			Long sts = sTs.get(i);
			Double serviceTimes = Double.valueOf(sts);
			serList.add(serviceTimes);
			new ArrayList<Date>();

		}

		myRegression = new WLSimpleRegression(noCasesList, serList);
		corrCoef = myRegression.getRegression();
		intercept = myRegression.getIntercept();
		rSquare = myRegression.getRSquare();
		slope = myRegression.getSlope();

		if (corrCoef < 0 || corrCoef > 0) {

			Pair<Double, Double> theSquare = new Pair<Double, Double>(corrCoef, rSquare);
			Pair<Double, Double> theSlope = new Pair<Double, Double>(intercept, slope);

			intermed = new Pair<Pair<Double, Double>, Pair<Double, Double>>(theSquare, theSlope);
		}
	}

	public Pair<Pair<Double, Double>, Pair<Double, Double>> getRegressionDetails() {
		return intermed;
	}

}
